#
# Copyright (C) 2005 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os.path
import shutil
import tempfile
import time
import locale
import gtk
import gtk.glade
import userprofile
import util
import aboutdialog
import saveconfirm
import gconfviewer
import fileviewer
from config import *
import debuglog
import errors

_ui_string = '''
<ui>
  <menubar name="Menubar">
    <menu action="ProfileMenu">
      <menuitem action="Save"/>
      <separator/>
      <menuitem action="Close"/>
    </menu>
    <menu action="EditMenu">
      <menuitem action="Delete"/>
    </menu>
    <menu action="HelpMenu">
      <menuitem action="About"/>
    </menu>
  </menubar>
</ui>
'''

def dprint (fmt, *args):
    debuglog.debug_log (False, debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL, fmt % args)

class ProfileModel (gtk.ListStore):
    (
        COLUMN_SOURCE,
        COLUMN_PATH,
        COLUMN_DESCRIPTION
    ) = range (3)

    def __init__ (self, profile):
        gtk.ListStore.__init__ (self, str, str, str)

        self.profile = profile
        self.reload ()

    def reload (self):
        dprint ("Reloading profile model")
        self.clear ()
        for (source_name, path) in self.profile.storage.list ():
            source = self.profile.get_source (source_name)
            if source is None:
                source = self.profile.get_delegate (source_name)
            
            dprint ("  source %s, path %s, description %s",
                    source_name,
                    path,
                    source.get_path_description (path))
            
            self.set (self.prepend (),
                      self.COLUMN_SOURCE,          source_name,
                      self.COLUMN_PATH,            path,
                      self.COLUMN_DESCRIPTION,     source.get_path_description (path))


class ProfileEditorWindow:
    def __init__ (self, profile_name, parent_window):
        self.profile_name = profile_name
        self.profile = userprofile.UserProfile (profile_name)
        self.storage = self.profile.storage
        self.last_save_time = 0

        self.window = gtk.Window (gtk.WINDOW_TOPLEVEL)
        self.window.set_title (_("Profile %s")%profile_name)
        self.window.set_icon_name ("sabayon")
        self.window.set_transient_for (parent_window)
        self.window.set_destroy_with_parent (True)
        self.window.set_default_size (480, 380)
        self.window.connect ("delete-event",
                             self.__handle_delete_event);

        self.main_vbox = gtk.VBox (False, 0)
        self.main_vbox.show ()
        self.window.add (self.main_vbox)

        self.__setup_menus ()

        self.scrolled = gtk.ScrolledWindow ()
        self.scrolled.set_policy (gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.scrolled.set_shadow_type (gtk.SHADOW_IN)
        self.scrolled.show ()
        self.main_vbox.pack_start (self.scrolled, True, True, 0)
        
        self.__setup_treeview ()

        self.window.show ()

        self.__set_needs_saving (False)

    def __reload_models (self):
        self.profile_model.reload ()
        dprint ("Reloaded models")

    def __set_needs_saving (self, needs_saving):
        if needs_saving:
            if not self.last_save_time:
                self.last_save_time = int (time.time ())
            self.save_action.set_sensitive (True)
        else:
            self.last_save_time = 0
            self.save_action.set_sensitive (False)
        self.__reload_models ()

    def __delete_currently_selected (self):
        (model, row) = self.treeview.get_selection ().get_selected ()
        if not row:
            return
    
        dprint ("Deleting '%s'", model[row][ProfileModel.COLUMN_PATH])

        self.storage.remove (model[row][ProfileModel.COLUMN_PATH])
        self.__set_needs_saving (True)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_key_press (self, treeview, event):
        if event.keyval in (gtk.keysyms.Delete, gtk.keysyms.KP_Delete):
            self.__delete_currently_selected ()
        
    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_save (self, action):
        self.storage.save ()
        self.__set_needs_saving (False)

    def __do_saveconfirm (self):
        if self.last_save_time:
            dialog = saveconfirm.SaveConfirmationAlert (self.window,
                                                        self.profile_name,
                                                        time.time () - self.last_save_time)
            response = dialog.run ()
            dialog.destroy ()

            if response == gtk.RESPONSE_CANCEL:
                return False
            if response == gtk.RESPONSE_YES:
                self.storage.save ()
                self.__set_needs_saving (False)
        
        return True
        
    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_close (self, action):
        if self.__do_saveconfirm ():
            self.window.destroy ()

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_delete_event (self, window, event):
        return not self.__do_saveconfirm ()

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_delete (self, action):
        self.__delete_currently_selected ()
        
    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_about (self, action):
        aboutdialog.show_about_dialog (self.window)

    def __add_widget (self, ui_manager, widget):
        self.main_vbox.pack_start (widget, False, False, 0)
        
    def __setup_menus (self):
        actions = [
            ("ProfileMenu", None, _("_Profile")),
            ("Save", gtk.STOCK_SAVE, _("_Save"), "<control>S", _("Save profile"), self.__handle_save),
            ("Close", gtk.STOCK_CLOSE, _("_Close"), "<control>W", _("Close the current window"), self.__handle_close),
            ("EditMenu", None, _("_Edit")),
            ("Delete", gtk.STOCK_DELETE, _("_Delete"), "<control>D", _("Delete item"), self.__handle_delete),
            ("HelpMenu", None, _("_Help")),
            ("About", gtk.STOCK_ABOUT, _("_About"), None, _("About Sabayon"), self.__handle_about),
        ]
        action_group = gtk.ActionGroup ("WindowActions")
        action_group.add_actions (actions)
        
        self.ui_manager = gtk.UIManager ()
        self.ui_manager.insert_action_group (action_group, 0)
        self.ui_manager.connect ("add-widget", self.__add_widget)
        self.ui_manager.add_ui_from_string (_ui_string)
        self.ui_manager.ensure_update ()

        self.window.add_accel_group (self.ui_manager.get_accel_group ())

        self.save_action = action_group.get_action ("Save")
        self.delete_action = action_group.get_action ("Delete")

    def __setup_treeview (self):
        self.profile_model = ProfileModel (self.profile)
        
        self.treeview = gtk.TreeView (self.profile_model)
        self.treeview.show ()
        self.scrolled.add (self.treeview)
        
        self.treeview.get_selection ().set_mode (gtk.SELECTION_SINGLE)
        self.treeview.get_selection ().connect ("changed",
                                                self.__treeview_selection_changed)

        self.treeview.set_headers_visible (False)

        c = gtk.TreeViewColumn (_("Description"),
                                gtk.CellRendererText (),
                                text = ProfileModel.COLUMN_DESCRIPTION)
        self.treeview.append_column (c)

        self.treeview.connect ("key-press-event", self.__handle_key_press)
        self.treeview.connect ("row-activated",   self.__handle_row_activation)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __treeview_selection_changed (self, selection):
        (model, row) = selection.get_selected ()
        if not row:
            self.delete_action.set_sensitive (False)
            return

        dprint ("Selected '%s'", model[row][ProfileModel.COLUMN_PATH])

        self.delete_action.set_sensitive (True)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_row_activation (self, treeview, tree_path, column):
        iter = self.profile_model.get_iter (tree_path)
        path = self.profile_model[iter][ProfileModel.COLUMN_PATH]
        description = self.profile_model[iter][ProfileModel.COLUMN_DESCRIPTION]
        source_name = self.profile_model[iter][ProfileModel.COLUMN_SOURCE]
        
        dprint ("Activating '%s'", path)
        
        extract_dir = tempfile.mkdtemp (prefix = "sabayon-temp-")
        self.storage.extract (path, extract_dir)
        extracted_path = os.path.join (extract_dir, path)

        if source_name == _("GConf"):
            viewer = gconfviewer.GConfViewer (extracted_path, description, self.window)
            viewer.connect ("destroy", lambda v, dir: shutil.rmtree (dir), extract_dir)
            viewer.show ()
        elif source_name == _("Files") or source_name == _("Panel"):
            viewer = fileviewer.FileViewer (extracted_path, description, self.window)
            viewer.connect ("destroy", lambda v, dir: shutil.rmtree (dir), extract_dir)
            viewer.show ()
        else:
            shutil.rmtree (extract_dir)



#
# Copyright (C) 2005 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import gobject
import exceptions, sys, os.path, ConfigParser, re, cPickle
import tempfile, types
import traceback
import errno
import time

try:
    import util
    import config
    import userprofile
    import dirmonitor
    import mozilla_bookmarks
    import debuglog
except:
    from sabayon import util
    from sabayon import config
    from sabayon import userprofile
    from sabayon import dirmonitor
    from sabayon import mozilla_bookmarks
    from sabayon import debuglog

class file_state:
    (UNKNOWN,
     VALID,
     NOT_FOUND,
     SYS_ERROR,
     PARSE_ERROR
     ) = range(5)


firefox_rel_path = ".mozilla/firefox"

profiles_ini_rel_path = os.path.join(firefox_rel_path, "profiles.ini")

sabayon_pref_rel_path = os.path.join(firefox_rel_path, "sabayon-prefs.js")
sabayon_mandatory_pref_rel_path = os.path.join(firefox_rel_path, "sabayon-mandatory-prefs.js")

sabayon_bookmark_rel_path = os.path.join(firefox_rel_path, "sabayon-bookmarks.html")
sabayon_mandatory_bookmark_rel_path = os.path.join(firefox_rel_path, "sabayon-mandatory-bookmarks.html")

# FIXME: these should be defined one place; see mozilla_bookmarks.py
LOG_OPERATION           = 0x00001
LOG_CHANGE              = 0x00002
LOG_IGNORED_CHANGE      = 0x00004
LOG_APPLY               = 0x00008
LOG_SYNC                = 0x00010
LOG_PARSE               = 0x00020
LOG_PREF                = 0x00040
LOG_FILE_CONTENTS       = 0x00080
LOG_DATA                = 0x00100
LOG_VERBOSE             = 0x10000

def dprint(mask, fmt, *args):
    # FIXME: before debuglog was introduced, we could use the mask to filter
    # which messages to log.  Now we don't use it anymore.  Is it still useful?
    # If you change this, synchronize it with mozilla_bookmarks.py
    debuglog.debug_log (False, debuglog.DEBUG_LOG_DOMAIN_MOZILLA_SOURCE, fmt % args)

bookmark_exclude_attrs = [re.compile("^id$"),
                          re.compile("^last_"),
                          re.compile("^add_"),
                          #re.compile("^icon$"),
                          ]

def filter_attr(attr, exclude):
    if not exclude:
        return False

    for regexp in exclude:
        if regexp.search(attr):
            return True
    return False

class MozillaChange(userprofile.ProfileChange):
    (
        CREATED,
        DELETED,
        CHANGED
    ) = range(3)
    
    def __init__ (self, source, delegate, type, key, value, event):
        userprofile.ProfileChange.__init__ (self, source, delegate)
        
        assert event == self.CREATED or \
               event == self.DELETED or \
               event == self.CHANGED
        
        self.type  = type
        self.key   = key
        self.value = value
        self.event = event
        self.attrs = {}

    def set_attr(self, attr, value):
        self.attrs[attr] = value

    def get_attr(self, attr):
        return self.attrs[attr]

    def get_type(self):
        return self.type

    def get_key(self):
        return self.key

    def get_value(self):
        return self.value

    def get_id(self):
        return self.key

    def get_short_description(self):
        if self.event == self.CREATED:
            return _("Mozilla key '%s' set to '%s'") % (self.key, self.value)
        elif self.event == self.DELETED:
            return _("Mozilla key '%s' unset") % self.key
        elif self.event == self.CHANGED:
            return _("Mozilla key '%s' changed to '%s'") % (self.key, self.value)
        else:
            raise ValueError

gobject.type_register(MozillaChange)

class MozillaDelegate(userprofile.SourceDelegate):
    def __init__ (self, source):
        dprint(LOG_OPERATION, "Delegate construction")
        userprofile.SourceDelegate.__init__ (self, "Firefox", source, ".mozilla")
        self.source = source
        self.delegate = self
        self.home_dir = util.get_home_dir()
        self.committed_prefs = {}
        self.committed_mandatory_prefs = {}
        self.committed_bookmarks = mozilla_bookmarks.BookmarkFolder("Null", None)
        self.committed_mandatory_bookmarks = mozilla_bookmarks.BookmarkFolder("Null", None)
        self.ini_file = None

    def get_full_path(self, path):
        return os.path.join(self.home_dir, path)        

    def get_profiles_ini_path(self):
        return self.get_full_path(profiles_ini_rel_path)

    def get_path_description (self, path):
        type = get_type_from_path(path)

        if   type == FirefoxProfileFile.TYPE_PREFS:
            return _("Web browser preferences")
        elif type == FirefoxProfileFile.TYPE_BOOKMARK:
            return _("Web browser bookmarks")
        elif type == FirefoxProfileFile.TYPE_PROFILE_INI:
            return _("Web browser profile list")
        else:
            # XXX
            basename = os.path.basename(path)
    
            if   basename.endswith("prefs.js"):
                return _("Web browser preferences")
            elif basename.endswith("bookmarks.html"):
                return _("Web browser bookmarks")
            else:
                return path

    def load_profiles_ini(self):
        if not self.ini_file:
            self.ini_file = FirefoxProfilesIni(self.home_dir, profiles_ini_rel_path)
        self.ini_file.read()

    def load_profiles(self):
        if self.ini_file:
            self.ini_file.load_profiles()

    def is_ini_file(self, rel_path):
        if rel_path == profiles_ini_rel_path:
            return True
        else:
            return False

    def is_profile_file(self, rel_path):
        if self.ini_file and self.ini_file.is_valid():
            rel_dir = os.path.dirname(rel_path)
            for profile in self.ini_file.get_profiles():
                profile_rel_dir = profile.get_rel_dir()
                if rel_dir.startswith(profile_rel_dir):
                    return profile
            return None
        else:
            return None

    def handle_change(self, change):
        rel_path = change.get_id()
        dprint (LOG_CHANGE, "handle_change: rel_path='%s'", rel_path)
        #
        # INI File
        #
        if self.is_ini_file(rel_path):
            dprint(LOG_CHANGE, "%s ini file: %s",
                   dirmonitor.event_to_string(change.event), rel_path)
            self.load_profiles_ini()

        #
        # Profile File
        #
        profile = self.is_profile_file(rel_path)
        if profile:
            dprint(LOG_CHANGE, "%s profile file: %s",
                   dirmonitor.event_to_string(change.event), rel_path)

            profile_file = profile.add_file(rel_path)
            profile_file_type = profile_file.get_type()

            if profile_file_type == FirefoxProfileFile.TYPE_PREFS:
                assert isinstance(profile_file, JavascriptPrefsFile)
                if change.event == dirmonitor.CREATED or \
                   change.event == dirmonitor.CHANGED:
                    cat_file(profile_file.get_full_path())
                    profile_file.read()
                    profile_file.emit_changes(self.source, self.delegate)
                    return True
                elif change.event == dirmonitor.DELETED:
                    # XXX - what to do here?
                    pass
                else:
                    raise ValueError
            elif profile_file_type == FirefoxProfileFile.TYPE_BOOKMARK:
                assert isinstance(profile_file, BookmarksFile)
                if change.event == dirmonitor.CREATED or \
                   change.event == dirmonitor.CHANGED:
                    profile_file.read()
                    profile_file.emit_changes(self.source, self.delegate)
                    return True
                elif change.event == dirmonitor.DELETED:
                    # XXX - what to do here?
                    pass
                else:
                    raise ValueError
            elif profile_file_type != FirefoxProfileFile.TYPE_UNKNOWN:
                if change.event == dirmonitor.CREATED or \
                   change.event == dirmonitor.CHANGED:
                    profile.add_file(rel_path)
                elif change.event == dirmonitor.DELETED:
                    profile.del_file(rel_path)
                else:
                    raise ValueError
            else:
                return True
                    
        #
        # Ignored Files
        #
        dprint(LOG_IGNORED_CHANGE, "IGNORED: %s", rel_path)
        return True

    def commit_change(self, change, mandatory = False):
        if isinstance(change, MozillaChange):
            dprint(LOG_CHANGE, "Commiting preference (mandatory = %s) key = %s value = %s event = %s",
                   mandatory, change.key, change.value, change.event)

            if mandatory:
                self.committed_mandatory_prefs[change.get_key()] = \
                    JavascriptPreference(change.get_type(), change.get_key(), change.get_value())
            else:
                self.committed_prefs[change.get_key()] = \
                    JavascriptPreference(change.get_type(), change.get_key(), change.get_value())
        elif isinstance(change, BookmarkChange):
            bookmark_path = change.get_bookmark_path()
            entry = change.get_entry()

            dprint(LOG_CHANGE, "Commiting bookmark (mandatory = %s) path = %s url = %s event = %s",
                   mandatory, entry.path_as_string(), entry.get_url(), change.event)

            if mandatory:
                self.committed_mandatory_bookmarks.add_path_entry(entry.path(), entry)
            else:
                self.committed_bookmarks.add_path_entry(entry.path(), entry)
                

    def start_monitoring (self):
        """Start monitoring for configuration changes."""
        # Read all files we are going to monitor so that when a change is
        # reported we have a "before" state to compare to and thus derive
        # what is different.
        dprint(LOG_OPERATION, "start_monitoring:")
        self.load_profiles_ini()
        self.load_profiles()

    def stop_monitoring (self):
        """Stop monitoring for configuration changes."""
        dprint(LOG_OPERATION, "stop_monitoring:")

    def sync_changes(self):
        """Ensure that all committed changes are saved to disk."""

        dprint(LOG_SYNC, "sync_changes: home_dir = %s", self.home_dir)
        ini = self.ini_file
        if ini.is_valid():
            default_profile = ini.get_default_profile()

            self.source.storage.add(ini.get_rel_path(), self.home_dir, self.name,
                                    {"file_type" : FirefoxProfileFile.TYPE_PROFILE_INI})

            if len(self.committed_prefs) > 0:
                pref_rel_path = sabayon_pref_rel_path
                dprint(LOG_SYNC, "sync_changes: storing committed_prefs to %s", pref_rel_path)
                pref_file = JavascriptPrefsFile(self.home_dir, pref_rel_path)
                pref_file.set_prefs(self.committed_prefs)
                pref_file.write()
                self.source.storage.add(pref_rel_path, self.home_dir, self.name,
                                         {"file_type" : FirefoxProfileFile.TYPE_PREFS,
                                          "mandatory"    : False})

            if len(self.committed_mandatory_prefs) > 0:
                pref_rel_path = sabayon_mandatory_pref_rel_path
                dprint(LOG_SYNC, "sync_changes: storing mandatory committed_prefs to %s", pref_rel_path)
                pref_file = JavascriptPrefsFile(self.home_dir, pref_rel_path)
                pref_file.set_prefs(self.committed_mandatory_prefs)
                pref_file.write()
                self.source.storage.add(pref_rel_path, self.home_dir, self.name,
                                         {"file_type" : FirefoxProfileFile.TYPE_PREFS,
                                          "mandatory"    : True})

            if len(self.committed_bookmarks.entries) > 0:
                bookmark_rel_path = sabayon_bookmark_rel_path
                dprint(LOG_SYNC, "sync_changes: storing committed_bookmarks to %s", bookmark_rel_path)
                bookmark_file = BookmarksFile(self.home_dir, bookmark_rel_path)
                bookmark_file.set_root(self.committed_bookmarks)
                bookmark_file.write(exclude_attrs=bookmark_exclude_attrs)
                self.source.storage.add(bookmark_rel_path, self.home_dir, self.name,
                                         {"file_type" : FirefoxProfileFile.TYPE_BOOKMARK,
                                          "mandatory"    : False})

            if len(self.committed_mandatory_bookmarks.entries) > 0:
                bookmark_rel_path = sabayon_mandatory_bookmark_rel_path
                dprint(LOG_SYNC, "sync_changes: storing mandatory committed_bookmarks to %s", bookmark_rel_path)
                bookmark_file = BookmarksFile(self.home_dir, bookmark_rel_path)
                bookmark_file.set_root(self.committed_mandatory_bookmarks)
                bookmark_file.write(exclude_attrs=bookmark_exclude_attrs)
                self.source.storage.add(bookmark_rel_path, self.home_dir, self.name,
                                         {"file_type" : FirefoxProfileFile.TYPE_BOOKMARK,
                                          "mandatory"    : True})

    def set_enforce_mandatory (self, enforce):
        # Nothing to do here
        pass

    def apply(self, is_sabayon_session):
        ini_files = []
        pref_files = []
        bookmark_files = []
        other_files = []
        
        dprint(LOG_APPLY, "apply: home_dir = %s", self.home_dir)
        storage_contents = self.source.storage.list(self.name)

        for source, path in storage_contents:
            attributes = self.source.storage.get_attributes(path)
            file_type = attributes.get("file_type", None)
            if file_type != None:
                file_type = int(file_type)
            else:
                file_type = get_type_from_path(path)

            if file_type == FirefoxProfileFile.TYPE_PROFILE_INI:
                ini_files.append(path)
            elif file_type == FirefoxProfileFile.TYPE_PREFS:
                pref_files.append(path)
            elif file_type == FirefoxProfileFile.TYPE_BOOKMARK:
                bookmark_files.append(path)
            elif file_type == FirefoxProfileFile.TYPE_UNKNOWN:
                other_files.append(path)
            else:
                raise ValueError
            
        dprint(LOG_APPLY, "apply: ini_files=%s pref_files=%s bookmark_files=%s other_files=%s",
               ini_files, pref_files, bookmark_files, other_files)

        # Profiles.ini file must be done first, if the target does not
        # exist then extract it from the profile.
        # Parse the profiles.ini file to learn the target profiles
        self.load_profiles_ini()
        if not self.ini_file.is_valid():
            dprint(LOG_APPLY, "apply: no valid ini file, extracting %s", profiles_ini_rel_path)
            if profiles_ini_rel_path in ini_files:
                self.source.storage.extract(profiles_ini_rel_path, self.home_dir, True)
                self.load_profiles_ini()
            else:
                dprint(LOG_APPLY, "apply: but there isn't an ini file in the profile!")

        # --------------------
        if sabayon_pref_rel_path in pref_files:
            dprint(LOG_APPLY, "extracting %s" % sabayon_pref_rel_path)
            self.source.storage.extract(sabayon_pref_rel_path, self.home_dir, True)
            apply_pref = JavascriptPrefsFile(self.home_dir, sabayon_pref_rel_path)
            apply_pref.read()
        else:
            apply_pref = None

        if sabayon_mandatory_pref_rel_path in pref_files:
            dprint(LOG_APPLY, "extracting %s" % sabayon_mandatory_pref_rel_path)
            self.source.storage.extract(sabayon_mandatory_pref_rel_path, self.home_dir, True)
            mandatory_apply_pref = JavascriptPrefsFile(self.home_dir, sabayon_mandatory_pref_rel_path)
            mandatory_apply_pref.read()
        else:
            mandatory_apply_pref = None

        # --------------------

        if sabayon_bookmark_rel_path in bookmark_files:
            dprint(LOG_APPLY, "extracting %s" % sabayon_bookmark_rel_path)
            self.source.storage.extract(sabayon_bookmark_rel_path, self.home_dir, True)
            apply_bookmark = BookmarksFile(self.home_dir, sabayon_bookmark_rel_path)
            apply_bookmark.read()
        else:
            apply_bookmark = None

        if sabayon_mandatory_bookmark_rel_path in bookmark_files:
            dprint(LOG_APPLY, "extracting %s" % sabayon_mandatory_bookmark_rel_path)
            self.source.storage.extract(sabayon_mandatory_bookmark_rel_path, self.home_dir, True)
            mandatory_apply_bookmark = BookmarksFile(self.home_dir, sabayon_mandatory_bookmark_rel_path)
            mandatory_apply_bookmark.read()
        else:
            mandatory_apply_bookmark = None


        # --------------------

        # Now merge the javascript pref files
        # XXX - iterate over all target profiles
        for profile in self.ini_file.get_profiles():
            dprint(LOG_APPLY, "apply: applying to profile %s", profile.attrs)
            profile_rel_dir = profile.get_rel_dir()

            # --------------------
            target_pref_rel_path = os.path.join(profile_rel_dir, "prefs.js")
            target_pref = JavascriptPrefsFile(self.home_dir, target_pref_rel_path)
            target_pref.read()

            if apply_pref:
                mandatory = False
                dprint(LOG_APPLY, "apply: applying src profile %s to target profile %s, mandatory=%s",
                       sabayon_pref_rel_path, target_pref_rel_path, mandatory)
                target_pref.merge(apply_pref, mandatory)

            if mandatory_apply_pref:
                mandatory = True
                dprint(LOG_APPLY, "apply: applying src profile %s to target profile %s, mandatory=%s",
                       sabayon_pref_rel_path, target_pref_rel_path, mandatory)
                target_pref.merge(mandatory_apply_pref, mandatory)

            if apply_pref or mandatory_apply_pref:
                target_pref.write()

            # --------------------
            target_bookmark_rel_path = os.path.join(profile_rel_dir, "bookmarks.html")
            target_bookmark = BookmarksFile(self.home_dir, target_bookmark_rel_path)
            target_bookmark.read()

            if apply_bookmark:
                mandatory = False
                dprint(LOG_APPLY, "apply: applying src profile %s to target profile %s, mandatory=%s",
                       sabayon_bookmark_rel_path, target_bookmark_rel_path, mandatory)
                target_bookmark.merge(apply_bookmark, mandatory)

            if mandatory_apply_bookmark:
                mandatory = True
                dprint(LOG_APPLY, "apply: applying src profile %s to target profile %s, mandatory=%s",
                       sabayon_bookmark_rel_path, target_bookmark_rel_path, mandatory)
                target_bookmark.merge(mandatory_apply_bookmark, mandatory)

            if apply_bookmark or mandatory_apply_bookmark:
                target_bookmark.write()

        # Finally extract any other file
        for path in other_files:
            attributes = self.source.storage.get_attributes(path)
            mandatory = attributes.get("mandatory", False)
            dprint(LOG_APPLY, "apply: extracting other file %s, mandatory=%s", path, mandatory)
            self.source.storage.extract(path, self.home_dir, mandatory)
            
        dprint(LOG_APPLY, "apply: finished")

def get_files_delegate(source):
    return MozillaDelegate(source)

#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------

# ------ Globals ------

# XXX - Warning: this regular expression is not perfectly robust
# the 1st parameter is expected to be a double quoted string without
# commas in it nor escaped double quotes. The parsing of the 2nd parameter
# should be robust. For our expected input it should be fine. Really
# robust parsing would require tokeninzing the expression.
pref_re = re.compile("(pref|user_pref|lock_pref)\s*\(\s*\"([^,\"]+)\s*\"\s*,\s*(.+?)\)\s*;\s*$", re.MULTILINE)

# ------ Excpetions ------

class FileNotFoundError(Exception):
    def __init__(self, filename):
        self.filename = filename
    def __str__(self):
        return _("File Not Found (%s)") % self.filename
    
class BadIniFileError(Exception):
    def __init__(self, problem):
        self.problem = problem
    def __str__(self):
        return self.problem
    

# ------ Class FirefoxProfileFile ------

class FirefoxProfileFile:
    (TYPE_UNKNOWN,
     TYPE_PROFILE_INI,
     TYPE_PREFS,
     TYPE_BOOKMARK
     ) = range(4)

    def __init__(self, home_dir, rel_path):
        self.home_dir = home_dir
        self.rel_path = rel_path
        self.attrs = {}
        self.file_type = get_type_from_path(rel_path)

    def get_full_path(self):
        return os.path.join(self.home_dir, self.rel_path)

    def get_rel_path(self):
        return self.rel_path

    def type_to_string(self, type):
        if type == FirefoxProfileFile.TYPE_UNKNOWN:
            return "UNKNOWN"
        if type == FirefoxProfileFile.TYPE_PROFILE_INI:
            return "PROFILE_INI"
        if type == FirefoxProfileFile.TYPE_PREFS:
            return "PREFS"
        if type == FirefoxProfileFile.TYPE_BOOKMARK:
            return "BOOKMARK"
        return "?"

    def get_type(self):
        return self.file_type

# ------ Class JavascriptPreference ------

class JavascriptPreference:
    def __init__(self, type, key, value):
        self.type = type
        self.key = key
        self.value = value

    def __eq__(self, other):
        if self.type  == other.type and \
           self.key   == other.key  and \
           self.value == other.value:
            return True
        else:
            return False

    def get_type(self):
        return self.type

    def get_key(self):
        return self.key

    def get_value(self):
        return self.value

# ------ Class JavascriptPrefsFile ------

class JavascriptPrefsFile(FirefoxProfileFile):
    def __init__(self, home_dir, rel_path):
        dprint(LOG_OPERATION, "JavascriptPrefsFile: created (%s)", rel_path)
        FirefoxProfileFile.__init__(self, home_dir, rel_path)
        self.file_state = file_state.UNKNOWN
        self.prefs = {}
        self.prev_prefs = {}

    def get_file_state(self):
        return self.file_state

    def write(self, full_path=None):
        if not full_path:
            full_path = self.get_full_path()
        dir = os.path.dirname(full_path)
        if not os.path.exists(dir):
            os.makedirs(dir)
        dprint(LOG_OPERATION, "JavascriptPrefsFile: writing file (%s)", full_path)
        fd = open(full_path, "w")
        fd.write('''
# Mozilla User Preferences

/*
 * Do not edit this file.
 * Created by %s, version %s
 */
 
''' % (config.PACKAGE, config.VERSION))

        keys = self.prefs.keys()
        keys.sort()
        for key in keys:
            pref = self.prefs[key]
            fd.write("%s(\"%s\", %s);\n" %
                     (pref.get_type(), pref.get_key(), pref.get_value()))

        fd.close()
        

    def merge(self, src, mandatory):
        for src_pref in src.prefs.values():
            src_key = src_pref.get_key()
            if not self.prefs.has_key(src_key) or mandatory:
                # XXX - should this just be a copy?
                self.prefs[src_key] = JavascriptPreference(src_pref.get_type(),
                                                           src_pref.get_key(),
                                                           src_pref.get_value())
                

    def read(self, full_path = None):
        self.prev_prefs = self.get_prefs()

        if not full_path:
            full_path = self.get_full_path()
        dprint(LOG_OPERATION, "read profile prefs (%s)", self.get_full_path())
        self.file_state = file_state.UNKNOWN
        try:
            fd = open(full_path)
        except IOError, e:
            if e.errno == errno.ENOENT:
                self.file_state = file_state.NOT_FOUND
                return
            else:
                self.file_state = file_state.SYS_ERROR
                raise

        self.filebuf = fd.read()
        fd.close()
        self.file_state = file_state.VALID

        self.kill_comments()
        self.parse()


    def emit_changes(self, source, delegate):
        cur_prefs = self.get_prefs()

        dc = util.DictCompare(self.prev_prefs, cur_prefs)
        dc.compare()
        cs = dc.get_change_set('a', 'b')

        _add = cs['add']
        _del = cs['del']
        _mod = cs['mod']

        def emit_changes(items, event):
            for key, pref in items:
                source.emit_change(
                    MozillaChange(source, delegate,
                                  pref.get_type(), pref.get_key(), pref.get_value(), event))

        emit_changes(_add.items(), MozillaChange.CREATED)
        emit_changes(_del.items(), MozillaChange.DELETED)
        emit_changes(_mod.items(), MozillaChange.CHANGED)

        

    def kill_comments(self):
        def not_in_string (regexp):
            double = '\\"(?:\\\\.|[^\"\\\\])*\\"'
            single = "\\'(?:\\\\.|[^\"\\\\])*\\'"
            return "(" + double + "|" + single + ")(?:" + regexp + ")"
        def match (matchobj):
            return matchobj.group(1)
        
            
        slash_comment_re = re.compile(not_in_string ("//.*$"), re.MULTILINE)
        hash_comment_re = re.compile(not_in_string ("#.*$"), re.MULTILINE)
        c_comment_re = re.compile(not_in_string ("/\*.*?\*/"), re.MULTILINE | re.DOTALL)

        self.filebuf = slash_comment_re.sub(match, self.filebuf)
        self.filebuf = hash_comment_re.sub(match, self.filebuf)
        self.filebuf = c_comment_re.sub(match, self.filebuf)

    def parse(self):
        start = 0;
        self.prefs = {}

        while 1:
            match = pref_re.search(self.filebuf, start)
            if match:
                type  = match.group(1)
                key   = match.group(2)
                value = match.group(3)
                dprint(LOG_PARSE, "(%d:%d) key='%s' value='%s'" %
                       (match.start(), match.end(), key, value))
                self.prefs[key] = JavascriptPreference(type, key, value)
                start = match.end()
            else:
                break

    def set_prefs(self, prefs):
        self.prefs = prefs.copy()

    def get_prefs(self):
        return self.prefs.copy()

    def dump_prefs(self):
        keys = self.prefs.keys()
        keys.sort()
        for key in keys:
            dprint(LOG_, "%s=%s" % (key, self.prefs[key]))


# ------ Class FirefoxProfile ------

class FirefoxProfile:
    def __init__(self, section, home_dir, rel_dir):
        self.section = section
        self.home_dir = home_dir
        self.rel_dir = rel_dir
        self.attrs = {}
        self.files = {}

    def set_attr(self, attr, value):
        self.attrs[attr] = value

    def get_attr(self, attr):
        return self.attrs[attr]

    def get_name(self):
        return self.get_attr("name")

    def get_default(self):
        return self.get_attr("default")

    def get_rel_dir(self):
        return os.path.join(self.rel_dir, self.get_dir())

    def get_dir(self):
        return self.get_attr("path")

    def add_file(self, rel_path):
        object = self.files.get(rel_path, None)
        if object:
            return(object)


        file_type = get_type_from_path(rel_path)

        if file_type == FirefoxProfileFile.TYPE_PREFS:
            object = JavascriptPrefsFile(self.home_dir, rel_path)
        elif file_type == FirefoxProfileFile.TYPE_BOOKMARK:
            object = BookmarksFile(self.home_dir, rel_path)
        else:
            object = FirefoxProfileFile(self.home_dir, rel_path)
        self.files[rel_path] = object
        return object

    def del_file(self, rel_path):
        if rel_path in self.files:
            del self.files[rel_path]

    def get_files_of_type(self, type):
        return [ file
                 for file in self.files.values() if file.get_type() == type ]


# ------ Class FirefoxProfilesIni ------

class FirefoxProfilesIni:
    def __init__(self, home_dir, rel_path):
        self.file_state = file_state.UNKNOWN
        self.default_profile = None
        self.profiles = {}
        self.ini = ConfigParser.ConfigParser()
        self.home_dir = home_dir
        self.rel_path = rel_path
        self.rel_dir = os.path.dirname(rel_path)

    def is_valid(self):
        return self.file_state == file_state.VALID

    def get_full_path(self, path):
        return os.path.join(self.home_dir, path)        

    def get_rel_dir(self):
        return self.rel_dir

    def get_rel_path(self):
        return self.rel_path

    def get_file_state(self):
        return self.file_state

    def load_profiles(self):
        dprint(LOG_OPERATION, "FirefoxProfilesIni.load_profiles()")
        for profile in self.get_profiles():
            profile_rel_dir = profile.get_rel_dir()

            pref_rel_path = os.path.join(profile_rel_dir, "prefs.js")
            dprint(LOG_OPERATION, "FirefoxProfilesIni.load_profiles() pref=%s", pref_rel_path)
            profile_file = profile.add_file(pref_rel_path)
            profile_file.read()

            bookmark_rel_path = os.path.join(profile_rel_dir, "bookmarks.html")
            dprint(LOG_OPERATION, "FirefoxProfilesIni.load_profiles() bookmark=%s", bookmark_rel_path)
            profile_file = profile.add_file(bookmark_rel_path)
            profile_file.read()



    def read(self):
        dprint(LOG_OPERATION, "FirefoxProfilesIni.read() path = %s",
               self.get_full_path(self.rel_path))
        self.profiles = {}

        try:
            if self.ini.read(self.get_full_path(self.rel_path)):
                self.file_state = file_state.VALID
            else:
                self.file_state = file_state.NOT_FOUND
        except ConfigParser.ParsingError, e:
            self.file_state = file_state.PARSE_ERROR
        

        dprint(LOG_PARSE, "FirefoxProfilesIni: after read, state = %s", self.file_state)
        if self.file_state != file_state.VALID:
            self.default_profile = None

        self.parse_sections()

    def parse_sections(self):
        profile_re = re.compile("^Profile(\d+)$")

        self.default_profile = None
        self.profiles = {}
        last_profile = None
        
        for section in self.ini.sections():
            dprint(LOG_PARSE, "parse_sections() section=%s", section)
            match = profile_re.match(section)
            if match:
                try:
                    default_profile = self.ini.get(section, "default")
                except ConfigParser.NoOptionError:
                    default_profile = None
                
                name = self.ini.get(section, "Name")
                if name in self.profiles:
                    raise BadIniFileError(_("duplicate name(%(name)s) in section %(section)s") %
                                          (name, section))
                profile = FirefoxProfile(section, self.home_dir, self.rel_dir)
                self.profiles[name] = profile
                for (key, value) in self.ini.items(section):
                    profile.set_attr(key, value)
                
                if default_profile:
                    if self.default_profile:
                        raise BadIniFileError(_("redundant default in section %s") %
                                              section)
                    self.default_profile = profile

                last_profile = profile

        if self.default_profile == None and len(self.profiles) == 1:
            # If there's only one profile, its the default even if it
            # doesn't have the Default=1 flag
            # Note: by default Firefox's auto-generated profile doesn't
            # have the Default= flag)
            self.default_profile = last_profile
            dprint(LOG_OPERATION, "defaulting profile to the only choice")
            
        
    def get_default_profile(self):
        if not self.default_profile:
            raise BadIniFileError(_("no default profile"))
        return self.default_profile

    def get_profiles(self, as_rel_dir=False):
        if as_rel_dir:
            return [ profile.get_rel_dir()
                     for profile in self.profiles.values() ]
        else:
            return self.profiles.values()

# ------ Class BookmarkChange ------

class BookmarkChange(userprofile.ProfileChange):
    (
        CREATED,
        DELETED,
        CHANGED
    ) = range(3)
    
    def __init__ (self, source, delegate, entry, event):
        userprofile.ProfileChange.__init__ (self, source, delegate)
        
        assert event == self.CREATED or \
               event == self.DELETED or \
               event == self.CHANGED
        
        self.entry = entry
        self.event = event
        self.attrs = {}

    def set_attr(self, attr, value):
        self.attrs[attr] = value

    def get_attr(self, attr):
        return self.attrs[attr]

    def get_bookmark_path(self):
        return self.entry.path_as_string()

    def get_entry(self):
        return self.entry

    def get_url(self):
        return self.entry.get_url()

    def get_id(self):
        return self.get_bookmark_path()

    def get_short_description(self):
        url = self.get_url()
        bookmark_path = self.get_bookmark_path()
        
        if self.event == self.CREATED:
            # XXX - don't test for url, use type
            if url:
                return _("Mozilla bookmark created '%s' -> '%s'") % (bookmark_path, url)
            else:
                return _("Mozilla bookmark folder created '%s'") % (bookmark_path)
        elif self.event == self.DELETED:
            if url:
                return _("Mozilla bookmark deleted '%s'") % (bookmark_path)
            else:
                return _("Mozilla bookmark folder deleted '%s'") % (bookmark_path)
        elif self.event == self.CHANGED:
            if url:
                return _("Mozilla bookmark changed '%s' '%s'") % (bookmark_path, url)
            else:
                return _("Mozilla bookmark folder changed '%s'") % (bookmark_path)
                
        else:
            raise ValueError

gobject.type_register(BookmarkChange)

# ------ Class BookmarksFile ------

class BookmarksFile(FirefoxProfileFile):
    def __init__(self, home_dir, rel_path):
        dprint(LOG_OPERATION, "BookmarksFile: created (%s)", rel_path)
        FirefoxProfileFile.__init__(self, home_dir, rel_path)
        self.parser = mozilla_bookmarks.BookmarkHTMLParser()
        self.root = mozilla_bookmarks.BookmarkFolder("Null", None)
        self.parser.set_root(self.root)
        self.prev_root = self.parser.get_root()
        self.file_state = file_state.UNKNOWN

    def get_full_path(self):
        return os.path.join(self.home_dir, self.rel_path)

    def get_rel_path(self):
        return self.rel_path

    def get_root(self):
        return self.root

    def set_root(self, root):
        self.root = root
        self.parser.set_root(self.root)

    def read(self):
        self.prev_root = self.parser.get_root()
        self.root = mozilla_bookmarks.BookmarkFolder("Null", None)
        self.parser.set_root(self.root)
        self.file_state = file_state.UNKNOWN
        full_path = self.get_full_path()
        dprint(LOG_OPERATION, "BookmarksFile: read (%s)", full_path)
        try:
            fd = open(full_path)
        except IOError, e:
            if e.errno == errno.ENOENT:
                self.file_state = file_state.NOT_FOUND
                return
            else:
                self.file_state = file_state.SYS_ERROR
                raise

        self.file_state = file_state.VALID
        self.parser.feed(fd.read())
        self.parser.close()
        self.root = self.parser.get_root()
        
    def write(self, full_path=None, exclude_attrs=None):
        if not full_path:
            full_path = self.get_full_path()
        dir = os.path.dirname(full_path)
        if not os.path.exists(dir):
            os.makedirs(dir)
        dprint(LOG_OPERATION, "MozillaBookmark: writing file (%s)", full_path)
        fd = open(full_path, "w")
        fd.write('''
<!DOCTYPE NETSCAPE-Bookmark-file-1>
<!-- This is an automatically generated file. (Created by %s, version %s)
     It will be read and overwritten.
     DO NOT EDIT! -->
<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8">
<TITLE>Bookmarks</TITLE>
<H1 LAST_MODIFIED="%.0f">Bookmarks</H1>

<DL><p>
''' % (config.PACKAGE, config.VERSION, time.time()))

        def visit(entry, type, path, data):
            indent = "    "
            level = len(path)

            if type == mozilla_bookmarks.TYPE_FOLDER:
                fd.write("%s<DT><H3" % (indent*level))
                for attr, value in entry.attrs.items():
                    if not filter_attr(attr, exclude_attrs):
                        fd.write(" %s=\"%s\"" % (attr, value))
                fd.write(">%s</H3>\n" % (entry.name))
                fd.write("%s<DL><p>\n" % (indent*level))
            elif type == mozilla_bookmarks.TYPE_BOOKMARK:
                fd.write("%s<DT><A" % (indent*level))
                for attr, value in entry.attrs.items():
                    if not filter_attr(attr, exclude_attrs):
                        fd.write(" %s=\"%s\"" % (attr, value))
                fd.write(">%s</A>\n" % (entry.name))
            elif type == mozilla_bookmarks.TYPE_FOLDER_END:
                fd.write("%s</DL><p>\n" % (indent*level))
            else:
                raise ValueError

        self.root.traverse(visit)
        fd.close()
        
    def convert_to_dict(self, root):
        result = {}

        def visit(entry, type, path, data):
            bookmark_path = entry.path_as_string()
            result[bookmark_path] = entry

        root.traverse(visit)
        return result
    
    def emit_changes(self, source, delegate):
        prev_dict = self.convert_to_dict(self.prev_root)
        cur_dict  = self.convert_to_dict(self.root)

        dc = util.DictCompare(prev_dict, cur_dict)
        dc.compare()
        cs = dc.get_change_set('a', 'b')

        _add = cs['add']
        _del = cs['del']
        _mod = cs['mod']

        def emit_changes(items, event):
            for bookmark_path, entry in items:
                source.emit_change(
                    BookmarkChange(source, delegate, entry, event))

        emit_changes(_add.items(), BookmarkChange.CREATED)
        emit_changes(_del.items(), BookmarkChange.DELETED)
        emit_changes(_mod.items(), BookmarkChange.CHANGED)


    def merge(self, src, mandatory):
        def visit(entry, type, path, data):
            if type == mozilla_bookmarks.TYPE_FOLDER_END:
                return
            dst_entry = self.root.lookup_path(path)
            if not dst_entry or mandatory:
                self.root.add_path_entry(path, entry)

        src.root.traverse(visit)


# ------ Utility Functions ------

def get_type_from_path(rel_path):
    basename = os.path.basename(rel_path)
    
    if basename == "prefs.js":
        return FirefoxProfileFile.TYPE_PREFS
    elif basename == "bookmarks.html":
        return FirefoxProfileFile.TYPE_BOOKMARK
    elif basename == "profiles.ini":
        return FirefoxProfileFile.TYPE_PROFILE_INI
    else:
        return FirefoxProfileFile.TYPE_UNKNOWN

def cat_file(path):
    if os.path.isfile(path):
        dprint(LOG_FILE_CONTENTS, "==== %s ====" % path)
        for line in open(path):
            dprint(LOG_FILE_CONTENTS, line.rstrip())
    else:
        dprint(LOG_FILE_CONTENTS, "WARNING, does not exist ==== %s ====" % path)



#-----------------------------------------------------------------------------

#
# Unit tests
#
def run_unit_tests():
    test_prefs = {'foo':'"bar"', 'uno':'1'}

    dprint(LOG_OPERATION, "In mozillaprofile tests")



#!/usr/bin/env python

#
# Copyright (C) 2006 Novell, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#

import gobject
import gtk
import gconf
import sys
import re

import globalvar
import lockdownbutton

class PessulusLockdownCombo:
    def __init__ (self, key, combo, type, value_list):
        self.key = key
        self.combo = combo
        self.type = type
        self.value_list = value_list;
        self.lockdownbutton = None

    def attach (combo, button, key, type, value_list):
        lockdown = PessulusLockdownCombo (key, combo, type, value_list)
        lockdown.lockdownbutton = lockdownbutton.PessulusLockdownButton.new_with_widget (button)
        # FIXME: connect to toggled ...
        lockdown.__connect_and_update ()
        return lockdown
    attach = staticmethod (attach)

    def __connect_and_update (self):
        self.lockdownbutton.connect ("toggled",
                                     self.__on_lockdownbutton_toggled)

        self.combo.connect ("changed", self.__on_combo_changed)
        self.combo.connect ("destroy", self.__on_destroyed)

        self.notify_id = globalvar.applier.notify_add (self.key,
                                                       self.__on_notified)
        self.update_state()

    def update_state(self):
        if self.isInt():
            (val, mandatory) = globalvar.applier.get_int (self.key)
            if val == None:
                val = "<not-in-list>"
            else:
                val = str (val)
        else:
            (val, mandatory) = globalvar.applier.get_string (self.key)
        
        index = 0
        for i in range (len(self.value_list)):
            if self.value_list[i] == val:
                index = i + 1
                break

        if self.combo.get_property ("active") != index:
            self.combo.set_property ("active", index)
            
        self.combo.set_sensitive (globalvar.applier.key_is_writable (self.key))
        if mandatory != self.lockdownbutton.get ():
            self.lockdownbutton.set (mandatory)

    def __on_notified(self, data):
        self.update_state()
        
    def getValue(self):
        idx = self.combo.get_property("active")
        if (idx < 1 or idx > len(self.value_list)):
            val = None
        else:
            val = self.value_list [idx - 1]
        return val
        
    def isUnset(self):
        if self.getValue() == None or self.getValue() == "":
            return True
        else:
            return False
        
    def isInt(self):
        return self.type == "int"

    def __do_change(self):
        if self.isUnset():
# FIXME: we should really use a 'tri-state' scheme & 'unset' the key            
            return
        if globalvar.applier and globalvar.applier.key_is_writable (self.key):
            if self.isInt():
                globalvar.applier.set_int (self.key, int(self.getValue()),
                                           self.lockdownbutton.get ())
            else:
                globalvar.applier.set_string (self.key, self.getValue(),
                                              self.lockdownbutton.get ())

    def __on_combo_changed (self, data):
        self.__do_change()

    def __on_lockdownbutton_toggled (self, lockdownbutton, mandatory):
        self.__do_change ()

    def __on_destroyed (self, combobox):
        if self.notify_id:
            if globalvar.applier:
                globalvar.applier.notify_remove (self.notify_id)
            self.notify_id = None



#!/usr/bin/env python

# vim: set ts=4 sw=4 et:

#
# Copyright (C) 2005 Vincent Untz <vuntz@gnome.org>
#
# This is based on code from gnome-menus (in menutreemodel.py)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#

import os
import gtk

def load_icon_from_path (icon_path):
    if os.path.isfile (icon_path):
        try:
            return gtk.gdk.pixbuf_new_from_file_at_size (icon_path, 24, 24)
        except:
            pass
    return None

def load_icon_from_data_dirs (icon_value):
    data_dirs = None
    if os.environ.has_key ("XDG_DATA_DIRS"):
        data_dirs = os.environ["XDG_DATA_DIRS"]
    if not data_dirs:
        data_dirs = "/usr/local/share/:/usr/share/"

    for data_dir in data_dirs.split (":"):
        retval = load_icon_from_path (os.path.join (data_dir, "pixmaps",
                                                    icon_value))
        if retval:
            return retval
        retval = load_icon_from_path (os.path.join (data_dir, "icons",
                                                    icon_value))
        if retval:
            return retval
    
    return None

def load_icon (icon_theme, icon_value):
    if not icon_value:
        return

    if os.path.isabs (icon_value):
        icon = load_icon_from_path (icon_value)
        if icon:
            return icon
        icon_name = os.path.basename (icon_value)
    else:
        icon_name = icon_value
    
    if icon_name.endswith (".png"):
        icon_name = icon_name[:-len (".png")]
    elif icon_name.endswith (".xpm"):
        icon_name = icon_name[:-len (".xpm")]
    elif icon_name.endswith (".svg"):
        icon_name = icon_name[:-len (".svg")]
    
    try:
        return icon_theme.load_icon (icon_name, 24, 0)
    except:
        return load_icon_from_data_dirs (icon_value)



#
# Copyright (C) 2005 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import time
import gobject
import gtk
import gtk.gdk
import pango
import util
import userprofile
import protosession
import changeswindow
import sessionwidget
import saveconfirm
import aboutdialog
import debuglog
import errors
from lockdownappliersabayon import LockdownApplierSabayon
from sabayon.lockdown import maindialog as lockdowndialog
from config import *

_ui_string = '''
<ui>
  <menubar name="Menubar">
    <menu action="ProfileMenu">
      <menuitem action="Save"/>
      <separator/>
      <menuitem action="Quit"/>
    </menu>
    <menu action="EditMenu">
      <menuitem action="Changes"/>
      <menuitem action="Lockdown"/>
      <menuitem action="EnforceMandatory"/>
    </menu>
    <menu action="HelpMenu">
      <menuitem action="About"/>
    </menu>
  </menubar>
</ui>
'''

def dprint (fmt, *args):
    debuglog.debug_log (False, debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL, fmt % args)

class ProfileChangesModel (gtk.ListStore):
    __gsignals__ = {
        "changed" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (userprofile.ProfileChange, ))
        }

    (
        COLUMN_CHANGE,
        COLUMN_IGNORE,
        COLUMN_MANDATORY,
        COLUMN_LOCK_PIXBUF,
        COLUMN_DESCRIPTION
    ) = range (5)

    def __init__ (self, profile):
        gtk.ListStore.__init__ (self, userprofile.ProfileChange, bool, bool, gtk.gdk.Pixbuf, str, str)

        icon_theme = gtk.icon_theme_get_default ()

        self.locked_pixbuf   = icon_theme.load_icon ("stock_lock",      16, 0)
        self.unlocked_pixbuf = icon_theme.load_icon ("stock_lock-open", 16, 0)

        self.profile = profile
        self.profile.connect ("changed", self.handle_profile_change)


    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL)
    def handle_profile_change (self, profile, new_change):
        default_mandatory = False
        ignore = new_change.get_ignore_default ()
        iter = self.find (new_change.get_source (), new_change.get_id ())
        old_change = None
        if iter:
            ignore    = self[iter][self.COLUMN_IGNORE]
            default_mandatory = self[iter][self.COLUMN_MANDATORY]
            old_change = self[iter][self.COLUMN_CHANGE]
            self.remove (iter)

        mandatory = new_change.get_mandatory ()
        if mandatory == None:
            mandatory = default_mandatory

        if mandatory:
            lock_pixbuf = self.locked_pixbuf
        else:
            lock_pixbuf = self.unlocked_pixbuf

        discard_change = False
        if old_change:
            discard_change = new_change.merge_old_change (old_change)

        if not discard_change:
            row = self.prepend ()
            self.set (row,
                      self.COLUMN_CHANGE,      new_change,
                      self.COLUMN_IGNORE,      ignore,
                      self.COLUMN_MANDATORY,   mandatory,
                      self.COLUMN_LOCK_PIXBUF, lock_pixbuf,
                      self.COLUMN_DESCRIPTION, new_change.get_short_description ())
        self.emit ("changed", new_change)

    def clear (self):
        gtk.ListStore.clear (self)
        self.emit ("changed", None)

    def find (self, source, id):
        iter = self.get_iter_first ()
        while iter:
            next = self.iter_next (iter)
            change = self[iter][self.COLUMN_CHANGE]
            if change.get_source () == source and \
               change.get_id ()     == id:
                return iter
            iter = next
        return None


gobject.type_register (ProfileChangesModel)

class SessionWindow:
    def __init__ (self, profile_name, profile_path, display_number):
        self.profile_name   = profile_name
        self.profile_path   = profile_path
        self.display_number = display_number

        self.profile = userprofile.UserProfile (profile_path)

        self.changes_model = ProfileChangesModel (self.profile)
        self.changes_model.connect ("changed", self.__changes_model_changed)
        self.changes_window = None
        self.lockdown_window = None

        self.last_save_time = 0

        self.window = gtk.Window ()
        self.window.set_title (_("Editing profile %s")%profile_name)
        self.window.set_icon_name ("sabayon")
        self.window.connect ("delete-event",
                             self.__handle_delete_event);

        self.box = gtk.VBox ()
        self.window.add (self.box)
        self.box.show ()

        self.__setup_menus ()
        self.__setup_session ()
        self.__setup_statusbar ()

        self.__set_needs_saving (False)


    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL)
    def __add_widget (self, ui_manager, widget):
        self.box.pack_start (widget, False, False, 0)

    def __setup_menus (self):
        actions = [
            ("ProfileMenu", None,            _("_Profile")),
            ("Save",        gtk.STOCK_SAVE,  _("_Save"),    "<control>S", _("Save profile"),             self.__handle_save),
            ("Quit",        gtk.STOCK_QUIT,  _("_Quit"),    "<control>Q", _("Close the current window"), self.__handle_quit),
            ("EditMenu",    None,            _("_Edit")),
            ("Changes",     gtk.STOCK_EDIT,  _("_Changes"), "<control>H", _("Edit changes"),             self.__handle_edit),
            ("Lockdown",    None,            _("_Lockdown"),"<control>L", _("Edit Lockdown settings"),   self.__handle_lockdown),
            ("HelpMenu",    None,            _("_Help")),
            ("About",       gtk.STOCK_ABOUT, _("_About"),   None,         _("About Sabayon"),            self.__handle_about),
        ]
        toggle_actions = [
            ("EnforceMandatory", None, _("Enforce Mandatory"), None, _("Enforce mandatory settings in the editing session"), self.__handle_enforce_mandatory, True),
        ]
        action_group = gtk.ActionGroup ("WindowActions")
        action_group.add_actions (actions)
        action_group.add_toggle_actions (toggle_actions)

        self.ui_manager = gtk.UIManager ()
        self.ui_manager.insert_action_group (action_group, 0)
        self.ui_manager.connect ("add-widget", self.__add_widget)
        self.ui_manager.add_ui_from_string (_ui_string)
        self.ui_manager.ensure_update ()

        self.window.add_accel_group (self.ui_manager.get_accel_group ())

        self.save_action = action_group.get_action ("Save")

    def __set_needs_saving (self, needs_saving):
        if needs_saving:
            if not self.last_save_time:
                self.last_save_time = int (time.time ())
            self.save_action.set_sensitive (True)
        else:
            self.last_save_time = 0
            self.save_action.set_sensitive (False)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL)
    def __changes_model_changed (self, model, change):
        self.__set_needs_saving (not model.get_iter_first () is None)

    def __do_save (self):
        # The changes model stores changes in reverse chronological order, so that
        # the latest changes are always visible at the top of the window.  To apply
        # the changes, we need them in chronological order.  So, we collect
        # all the changes and reverse that list before committing the changes.

        all_changes = []
        iter = self.changes_model.get_iter_first ()
        while iter:
            change    = self.changes_model[iter][ProfileChangesModel.COLUMN_CHANGE]
            ignore    = self.changes_model[iter][ProfileChangesModel.COLUMN_IGNORE]
            mandatory = self.changes_model[iter][ProfileChangesModel.COLUMN_MANDATORY]
            all_changes.append ((change, ignore, mandatory))
            iter = self.changes_model.iter_next (iter)

        all_changes.reverse ()

        # Commit the changes!

        for (change, ignore, mandatory) in all_changes:
            if not ignore:
                dprint ("Committing: %s, mandatory = %s", change.get_id (), mandatory)
                try:
                    change.get_source ().commit_change (change, mandatory)
                except:
                    errors.errors_log_recoverable_exception (debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL,
                                                             "got an exception while commiting a change")

        # Done

        self.changes_model.clear ()
        self.profile.sync_changes ()

    def __do_saveconfirm (self):
        if self.last_save_time:
            dialog = saveconfirm.SaveConfirmationAlert (self.window,
                                                        self.profile_name,
                                                        time.time () - self.last_save_time)
            response = dialog.run ()
            dialog.destroy ()

            if response == gtk.RESPONSE_CANCEL or \
               response == gtk.RESPONSE_DELETE_EVENT:
                return False
            if response == gtk.RESPONSE_YES:
                self.__do_save ()

        return True

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_save (self, action):
        self.__do_save ()

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_quit (self, action):
        if self.__do_saveconfirm ():
            self.window.destroy ()

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_delete_event (self, window, event):
        return not self.__do_saveconfirm ()

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_about (self, action):
        aboutdialog.show_about_dialog (self.window)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_edit (self, action):
        if not self.changes_window:
            self.changes_window = changeswindow.ChangesWindow (self.changes_model,
                                                               self.profile_name,
                                                               self.window)
            self.changes_window.window.connect ("delete-event",
                                                gtk.Widget.hide_on_delete)
        self.changes_window.window.present ()

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_lockdown (self, action):
        if not self.lockdown_window:
            debuglog.uprint ("Creating new Lockdown window")
            applier = LockdownApplierSabayon (self.profile, self.changes_model)
            self.lockdown_window = lockdowndialog.PessulusMainDialog (applier, False)
            self.lockdown_window.window.set_title (_("Lockdown settings for %s")%self.profile_name)
        else:
            debuglog.uprint ("Presenting existing Lockdown window")

        self.lockdown_window.window.present ()

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_enforce_mandatory (self, action):
        active = action.get_active ()
        debuglog.uprint ("Setting enforce_mandatory to %s", active)
        self.profile.set_enforce_mandatory (active)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL)
    def __session_finished (self, session):
        self.window.destroy ()

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL)
    def __session_mapped (self, session_widget, event):
        dprint ("Session widget mapped; starting prototype session")
        self.session_widget.disconnect (self.mapped_handler_id)
        self.mapped_handler_id = 0
        self.session.connect ("finished", self.__session_finished)
        self.session.start (str (self.session_widget.session_window.xid))
        return False

    def __setup_session (self):
        self.session = protosession.ProtoSession (self.profile_name, self.display_number)

        try:
            self.session.apply_profile ()
        except errors.RecoverableApplyErrorException, e:
            errors_log_recoverable_exception (e)
            dialog = gtk.MessageDialog (parent = None,
                                        flags = gtk.DIALOG_MODAL,
                                        type = gtk.MESSAGE_ERROR,
                                        buttons = gtk.BUTTONS_NONE,
                                        message_format = _("There was a recoverable error while applying the "
                                                           "user profile '%s'.  You can report this error now "
                                                           "or try to continue editing the user profile."))
            (REPORT, CONTINUE) = range (2)
            dialog.add_button (_("_Report this error"), REPORT)
            dialog.add_button (_("_Continue editing"), CONTINUE)
            response = dialog.run ()
            dialog.destroy ()

            if response == REPORT:
                raise # the toplevel will catch the RecoverableApplyErrorException and exit

        except errors.FatalApplyErrorException, e:
            raise # FIXME: do we need any special processing?  Should we give the user
                  # the option of continuing editing?

        self.profile.start_monitoring ()
        screen = gtk.gdk.screen_get_default ()
        width  = (screen.get_width ()  * 3) / 4
        height = (screen.get_height () * 3) / 4

        dprint ("Creating %dx%d session widget", width, height)

        self.session_widget = sessionwidget.SessionWidget (width, height)
        self.box.pack_start (self.session_widget, True, True)
        self.mapped_handler_id = self.session_widget.connect ("map-event", self.__session_mapped)
        self.session_widget.show ()

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL)
    def __update_statusbar (self, model, change):
        if change:
            self.statusbar.pop (self.status_context_id)
            self.statusbar.push (self.status_context_id,
                                 change.get_short_description ())

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL)
    def __update_resize_grip (self, window, event):
        if event.changed_mask & gtk.gdk.WINDOW_STATE_MAXIMIZED:
            if event.new_window_state & gtk.gdk.WINDOW_STATE_MAXIMIZED:
                self.statusbar.set_has_resize_grip (False)
            else:
                self.statusbar.set_has_resize_grip (True)

    def __setup_statusbar (self):
        self.statusbar = gtk.Statusbar ()
        self.box.pack_start (self.statusbar, False, False)
        self.statusbar.show ()

        self.status_context_id = self.statusbar.get_context_id ("sabayon-change")

        self.changes_model.connect ("changed", self.__update_statusbar)

        self.window.connect ("window-state-event", self.__update_resize_grip)



#
# Copyright (C) 2005 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os.path
import pwd
import gtk
import gtk.glade
import userdb
import errors
import debuglog

from config import *

class UsersModel (gtk.ListStore):
    (
        COLUMN_NAME,
        COLUMN_USER,
        COLUMN_APPLY
    ) = range (3)

    def __init__ (self, db, profile):
        gtk.ListStore.__init__ (self, str, str, bool)
        for user in db.get_users ():
            pw = pwd.getpwnam (user)

            name = None
            if pw.pw_gecos:
                name = pw.pw_gecos.split (",")[0]
            if not name:
                name = user

            row = self.append ()
            self.set (row,
                      self.COLUMN_NAME,  name,
                      self.COLUMN_USER,  user,
                      self.COLUMN_APPLY, profile == db.get_profile (user, False, True))

class UsersDialog:
    def __init__ (self, profile, parent):
        self.profile = profile
        self.userdb = userdb.get_database ()

        apply_to_all = self.userdb.get_default_profile (False) == profile
        
        glade_file = os.path.join (GLADEDIR, "sabayon.glade")
        self.xml = gtk.glade.XML (glade_file, "users_dialog", PACKAGE)

        self.dialog = self.xml.get_widget ("users_dialog")
        self.dialog.set_transient_for (parent)
        self.dialog.set_default_response (gtk.RESPONSE_CLOSE)
        self.dialog.set_icon_name ("sabayon")
        self.dialog.set_title (_("Users for profile %s")%profile)

        self.close_button = self.xml.get_widget ("users_close_button")

        self.help_button = self.xml.get_widget ("users_help_button")
        self.help_button.hide ()

        self.all_check = self.xml.get_widget ("users_all_check")
        self.all_check.set_active (apply_to_all)
        self.all_check.connect ("toggled", self.__all_check_toggled)

        self.users_model = UsersModel (self.userdb, self.profile)
        
        self.users_list_scroll = self.xml.get_widget ("users_list_scroll")
        self.users_list = self.xml.get_widget ("users_list")
        self.users_list.set_model (self.users_model)
        self.users_list.set_sensitive (not apply_to_all)

        c = gtk.TreeViewColumn (_("Name"),
                                gtk.CellRendererText (),
                                text = UsersModel.COLUMN_NAME)
        c.set_sort_column_id(UsersModel.COLUMN_NAME)        
        self.users_list.append_column (c)
        self.users_model.set_sort_column_id(UsersModel.COLUMN_NAME, gtk.SORT_ASCENDING)


        toggle = gtk.CellRendererToggle ()
        toggle.connect ("toggled", self.__on_use_toggled)
        c = gtk.TreeViewColumn (_("Use This Profile"))
        c.pack_start (toggle, False)
        c.set_attributes (toggle, active = UsersModel.COLUMN_APPLY)
        self.users_list.append_column (c)
        
        response = self.dialog.run ()
        self.dialog.hide ()

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __on_use_toggled (self, toggle, path):
        iter = self.users_model.get_iter_from_string (path)
        apply = self.users_model.get_value (iter, UsersModel.COLUMN_APPLY)

        apply = not apply

        self.users_model.set (iter, UsersModel.COLUMN_APPLY, apply)

        username = self.users_model.get_value (iter, UsersModel.COLUMN_USER)
        
        if apply:
            self.userdb.set_profile (username, self.profile)
        else:
            self.userdb.set_profile (username, None)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __all_check_toggled (self, toggle):
        apply_to_all = self.all_check.get_active ()
        self.users_list.set_sensitive (not apply_to_all)

        if apply_to_all:
            self.userdb.set_default_profile (self.profile)
        else:
            self.userdb.set_default_profile (None)

if __name__ == "__main__":
    import util

    util.init_gettext ()
    
    d = UsersDialog ("foo", None)



#
# Copyright (C) 2005 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os
import os.path
import gobject
import gnomevfs
import util
import fnmatch
import debuglog
import urllib

N_WATCHES_LIMIT = 200

CHANGED = gnomevfs.MONITOR_EVENT_CHANGED
DELETED = gnomevfs.MONITOR_EVENT_DELETED
CREATED = gnomevfs.MONITOR_EVENT_CREATED

def dprint (fmt, *args):
    debuglog.debug_log (False, debuglog.DEBUG_LOG_DOMAIN_DIR_MONITOR, fmt % args)

def event_to_string (event):
    if event == CHANGED:
        return "changed"
    elif event == DELETED:
        return "deleted"
    elif event == CREATED:
        return "created"
    else:
        return "invalid"

class DirectoryMonitor:
    def __init__ (self, directory, callback, data = None):
        self.directory = directory
        self.callback = callback
        self.data = data
	self.watches = {} # maps filename => gnome_vfs_monitor handle
        self.too_many_watches = False
        self.dirs_to_ignore = []
        self.files_to_ignore = []

    def set_directories_to_ignore (self, dirs):
        assert len (self.watches) == 0
        self.dirs_to_ignore = dirs
        dprint ("Ignoring directories %s", self.dirs_to_ignore)

    def set_files_to_ignore (self, files):
        assert len (self.watches) == 0
        self.files_to_ignore = files
        dprint ("Ignoring files %s", self.files_to_ignore)

    #
    # call the user level processing
    #
    def __invoke_callback (self, path, event):
        dprint ("Invoking callback for %s - %s", path, event_to_string (event))
        if self.data:
            self.callback (path, event, self.data)
        else:
            self.callback (path, event)

    #
    # Processing of a gnomevfs callback
    #
    def __handle_gnomevfs_event (self, dir_uri, file_uri, event):
        if event == CHANGED or event == DELETED or event == CREATED:
            # Strip 'file://' and replace %xx with equivalent characters.
            path = file_uri [7:]
            path = urllib.unquote (path)

            # Strip trailing '/'.
            path = os.path.normpath (path)
            
            dprint ("Got gnomevfs event '%s' on '%s'", event_to_string (event), path)

            if not self.__should_ignore_dir (path) and \
               not self.__should_ignore_file (path):
                self.__invoke_callback (path, event)

                if event == CREATED and os.path.isdir (path):
                    self.__monitor_dir_recurse (path, True)
                elif event == DELETED:
                    if path != self.directory and self.watches.has_key (path):
                        dprint ("Deleting watch for '%s' since it got deleted", path)
                        gnomevfs.monitor_cancel (self.watches [path])
                        del self.watches[path]
                        if len (self.watches) < N_WATCHES_LIMIT:
                            self.too_many_watches = False
            else:
                dprint ("Not calling callback for '%s' nor recursing in it since it is an ignored dir/file", path)

    def __should_ignore_dir (self, dir):
        dir = os.path.normpath (dir)
        
        for ignore_dir in self.dirs_to_ignore:
            ignore_path = os.path.normpath (os.path.join (self.directory, ignore_dir))

            if fnmatch.fnmatch (dir, ignore_path):
                return True

        parent = os.path.dirname (dir)
        if parent != dir:
            return self.__should_ignore_dir (parent)
        else:
            return False
    
    def __should_ignore_file (self, file):
        file = os.path.normpath (file)
        
        for ignore_file in self.files_to_ignore:
            ignore_path = os.path.normpath (os.path.join (self.directory, ignore_file))
            if fnmatch.fnmatch (file, ignore_path):
                return True
        return self.__should_ignore_dir (os.path.dirname (file))

    def __monitor_dir (self, dir):
        dir = os.path.normpath (dir)
        
        if len (self.watches) >= N_WATCHES_LIMIT:
            if not self.too_many_watches:
                print "Too many directories to watch on %s" % (self.directory)
                self.too_many_watches = True
            return

        try:
            self.watches [dir] = gnomevfs.monitor_add (dir, gnomevfs.MONITOR_DIRECTORY, self.__handle_gnomevfs_event)
            dprint ("Added directory watch for '%s'", dir)
        except:
            print ("Failed to add monitor for %s") % (dir)
	    util.print_exception ()

    def __monitor_dir_recurse (self, dir, new_dir = False):
        if self.too_many_watches:
            dprint ("Skipping recursion on '%s', as there are already too many watches", dir)
            return

        if self.__should_ignore_dir (dir):
            dprint ("Skipping recursion on '%s'; it is an ignored directory", dir)
            return

        if dir != self.directory:
            self.__monitor_dir (dir)

        for entry in os.listdir (dir):
            path = os.path.join (dir, entry)
            if self.__should_ignore_dir (path) or \
               self.__should_ignore_file (path):
                dprint ("Skipping callback and recursion on '%s'; it is an ignored file/dir", path)
                continue

            self.__invoke_callback (path, CREATED)

            if os.path.isdir (path):
                self.__monitor_dir_recurse (path, new_dir)
                    
    def start (self):
        dprint ("Starting to recursively monitor '%s'", self.directory)
        self.__monitor_dir (self.directory)
        self.__monitor_dir_recurse (self.directory)
        dprint ("Ending recursive scan of '%s'; all monitors are in place now", self.directory)

    def stop (self):
        dprint ("Stopping recursive monitoring of '%s'", self.directory)

        for path in self.watches:
            gnomevfs.monitor_cancel (self.watches [path])

def run_unit_tests ():
    import tempfile
    import shutil

    temp_path = tempfile.mkdtemp (prefix = "test-monitor-")

    def handle_change (path, event, data):
        (expected, main_loop) = data
        if len (expected) > 0:
            i = 0
            for (expected_path, expected_event) in expected:
                if expected_path == path and expected_event == event:
                    break
                i += 1
            if i < len (expected):
                del expected[i]
        if len (expected) == 0:
            main_loop.quit ()

    def expect (expected, path, event):
        expected.append ((path, event))

    main_loop = gobject.MainLoop ()

    expected = []
    def should_not_be_reached (expected):
        for (path, event) in expected:
            print ("Expected event: %s %s") % (path, event_to_string (event))
        assert False
        return True
    timeout = gobject.timeout_add (5 * 1000, should_not_be_reached, expected)

    monitor = DirectoryMonitor (temp_path, handle_change, (expected, main_loop))
    monitor.set_directories_to_ignore (["bar"])
    monitor.set_files_to_ignore (["foobar/foo/foo.txt"])
    monitor.start ()

    expect (expected, os.path.join (temp_path, "foo.txt"), CREATED)
    f = file (os.path.join (temp_path, "foo.txt"), "w")
    f.close ()

    # ignored
    # expect (expected, os.path.join (temp_path, "bar"), CREATED)
    os.mkdir (os.path.join (temp_path, "bar"))

    expect (expected, os.path.join (temp_path, "foobar"), CREATED)
    expect (expected, os.path.join (temp_path, "foobar/foo"), CREATED)
    expect (expected, os.path.join (temp_path, "foobar/foo/bar"), CREATED)
    expect (expected, os.path.join (temp_path, "foobar/foo/bar/foo"), CREATED)
    expect (expected, os.path.join (temp_path, "foobar/foo/bar/foo/bar"), CREATED)
    os.makedirs (os.path.join (temp_path, "foobar/foo/bar/foo/bar"))

    # ignored:
    # expect (expected, os.path.join (temp_path, "foobar/foo/foo.txt"), CREATED)
    f = file (os.path.join (temp_path, "foobar/foo/foo.txt"), "w")
    f.close ()

    main_loop.run ()
    
    expect (expected, os.path.join (temp_path, "foobar/foo/bar/foo/bar"), DELETED)
    expect (expected, os.path.join (temp_path, "foobar/foo/bar/foo"), DELETED)
    expect (expected, os.path.join (temp_path, "foobar/foo/bar"), DELETED)
    # ignored:
    # expect (expected, os.path.join (temp_path, "foobar/foo/foo.txt"), DELETED)
    expect (expected, os.path.join (temp_path, "foobar/foo"), DELETED)
    expect (expected, os.path.join (temp_path, "foobar"), DELETED)
    
    # FIXME: we should be getting this event, but we don't seem to be
    # expect (expected, os.path.join (temp_path, "foo.txt"), DELETED)
    
    # ignore:
    # expect (expected, os.path.join (temp_path, "bar"), DELETED)
    expect (expected, temp_path, DELETED)

    shutil.rmtree (temp_path, True)

    main_loop.run ()

    gobject.source_remove (timeout)

    monitor.stop ()



#
# Copyright (C) 2005 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import gtk
import errors
import debuglog
from config import *

#
# Of course, this would all be so much easier if
# gtk_show_about_dialog() was wrapped
#

global_about_dialog = None

def show_about_dialog (parent_window = None):

    if parent_window:
        about_dialog = parent_window.get_data ("gtk-about-dialog")
    else:
        global global_about_dialog
        about_dialog = global_about_dialog

    if about_dialog:
        about_dialog.present ()
        return

    authors = [
        "Daniel Veillard <veillard@redhat.com>",
        "John Dennis <jdennis@redhat.com>",
        "Mark McLoughlin <markmc@redhat.com>"
    ]

    # documenters = [
    # ]

    about_dialog = gtk.AboutDialog ()

    if parent_window:
        about_dialog.set_transient_for (parent_window)
    about_dialog.set_destroy_with_parent (True)
    about_dialog.set_icon_name ("sabayon")

    about_dialog.set_name               (PACKAGE)
    about_dialog.set_version            (VERSION)
    about_dialog.set_copyright          ("(C) 2005 Red Hat, Inc.")
    about_dialog.set_website            ("http://www.gnome.org/projects/sabayon")
    about_dialog.set_comments           (_("Program to establish and edit profiles for users"))
    about_dialog.set_authors            (authors)
    about_dialog.set_logo_icon_name     ("sabayon")
    about_dialog.set_translator_credits (_("translator-credits"))

    # about_dialog.set_documenters        (documenters)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def handle_delete (about, event):
        about.hide ()
        return True
    about_dialog.connect ("delete-event", handle_delete)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def handle_response (about, response):
        about.hide ()
    about_dialog.connect ("response", handle_response)

    about_dialog.present ()

    if parent_window:
        parent_window.set_data ("gtk-about-dialog", about_dialog)
    else:
        global_about_dialog = about_dialog



#
# Copyright (C) 2005 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

# XXX - TODO:
# add support for DD tags
# add support for HR format tags

import sys
import os
import re
from HTMLParser import HTMLParser

try:
    import util
    import config
except:
    from sabayon import util
    from sabayon import config

debug = 0
indent = '    '

bookmark_separator = "/"
TYPE_FOLDER     = 1
TYPE_BOOKMARK   = 2
TYPE_FOLDER_END = 3

tag_info_dict = {
    'dt' : {'implicit_close_event' : ['begin'],
            'implicit_close_scope' : ['dl'],
            'implicit_close_tags'  : ['dt', 'dd']},
    'dd' : {'implicit_close_event' : ['begin'],
            'implicit_close_scope' : ['dl'],
            'implicit_close_tags'  : ['dd']},
    'dl' : {'implicit_close_event' : ['begin', 'end'],
            'implicit_close_scope' : ['dl'],
            'implicit_close_tags'  : ['dt', 'dd']},
    'p'  : {'simple_tag'           : True},
    'hr' : {'simple_tag'           : True},
}

# FIXME: these should be defined one place; see mozillasource.py
LOG_OPERATION           = 0x00001
LOG_CHANGE              = 0x00002
LOG_IGNORED_CHANGE      = 0x00004
LOG_APPLY               = 0x00008
LOG_SYNC                = 0x00010
LOG_PARSE               = 0x00020
LOG_PREF                = 0x00040
LOG_FILE_CONTENTS       = 0x00080
LOG_DATA                = 0x00100
LOG_VERBOSE             = 0x10000

def dprint(mask, fmt, *args):
    # FIXME: before debuglog was introduced, we could use the mask to filter
    # which messages to log.  Now we don't use it anymore.  Is it still useful?
    # If you change this, synchronize it with mozillasource.py
    debuglog.debug_log (False, debuglog.DEBUG_LOG_DOMAIN_MOZILLA_SOURCE, fmt % args)

class Bookmark:
    def __init__(self, folder, name):
        self.folder = folder
        self.name = name
        self.attrs = {}

    def get_attr(self, name):
        return self.attrs.get(name, None)

    def get_url(self):
        return self.attrs.get("href", None)

    def path(self):
        path = self.folder.path()
        path.append(self)
        return path

    def path_as_names(self, join=None):
        path = self.folder.path_as_names()
        path.append(self.name)
        if join == None:
            return path
        else:
            return join.join(path)

    def path_as_string(self):
        return self.path_as_names(bookmark_separator)                       

class BookmarkFolder:
    def __init__(self, name, parent):
        self.reset(name, parent)

    def reset(self, name, parent):
        self.name = name
        self.parent = parent
        self.attrs = {}
        self.entries = []

    def entry_index(self, entry):
        n_entries = len(self.entries)
        i = 0
        while (i < n_entries):
            if self.entries[i] == entry:
                return i
            i += 1
        return None

    def add_entry(self, entry):
        self.entries.append(entry)
        return entry

    def add_folder(self, folder):
        if not isinstance(folder, BookmarkFolder):
            folder = BookmarkFolder(folder, self)
        self.entries.append(folder)
        return folder

    def lookup_folder(self, folder):
        for entry in self.entries:
            if isinstance(entry, BookmarkFolder):
                if entry == folder:
                    return entry
        return None

    def add_bookmark(self, bookmark):
        if not isinstance(bookmark, Bookmark):
            bookmark = Bookmark(self, bookmark)
        self.entries.append(bookmark)
        return bookmark


    def lookup_bookmark(self, bookmark):
        for entry in self.entries:
            if isinstance(entry, Bookmark):
                if entry == bookmark:
                    return entry
        return None

    def lookup_path(self, path):
        path_len = len(path)
        i = 0
        folder = self
        while i < path_len - 1:
            folder = folder.lookup_folder(path[i])
            if not folder:
                return None
            i += 1
        entry_index = folder.entry_index(path[i])
        if entry_index == None:
            return None
        else:
            return folder.entries[entry_index]
                    

    def add_path_entry(self, path, entry):
        path_len = len(path)
        i = 0
        parent = folder = self
        while i < path_len - 1:
            folder = parent.lookup_folder(path[i])
            if not folder:
                folder = parent.add_folder(path[i])
            parent = folder
            i += 1
        if folder.entry_index(path[i]) == None:
            folder.add_entry(path[i])
        

    def set_attr(self, name, value):
        self.attrs[name] = value

    def get_attr(self, name):
        return self.attrs.get(name, None)

    def get_url(self):
        return self.attrs.get("href", None)

    def path(self):
        path = [self]
        folder = self
        parent = self.parent
        while parent:
            path.append(parent)
            parent = parent.parent
        path.reverse()
        return path
    
    def path_as_names(self, join=None):
        path = self.path()
        path = [ p.name for p in path ]
        if join == None:
            return path
        else:
            return join.join(path)
        
    def path_as_string(self):
        return self.path_as_names(bookmark_separator)                       

    def _traverse(self, visit_func, path, data):
        assert isinstance(self, BookmarkFolder)

        path.append(self)
        for entry in self.entries:
            if isinstance(entry, BookmarkFolder):
                visit_func(entry, TYPE_FOLDER, path, data)
                entry._traverse(visit_func, path, data)
            elif isinstance(entry, Bookmark):
                visit_func(entry, TYPE_BOOKMARK, path, data)
            else:
                raise ValueError
        path.pop()
        visit_func(self, TYPE_FOLDER_END, path, data)

    def traverse(self, visit_func, data=None):
        path = []
        self._traverse(visit_func, path, data)


    def find_bookmark(self, name):
        result = []

        def visit(entry, type, path, data):
            if type == TYPE_BOOKMARK:
                if entry.name == name:
                    result.append(entry)

        self.traverse(visit)
        return result
    
# ----------------------------------

class HTMLTag:
    def __init__(self, tag):
        self.tag = tag
        self.attrs = {}
        self.data = ""

class BookmarkHTMLParser(HTMLParser):
    def __init__(self, root=None):
        HTMLParser.__init__(self)
        self.stack = [HTMLTag("None")]
        self.folder_root = root
        self.cur_folder = self.folder_root

    def set_root(self, root):
        self.folder_root = root

    def get_root(self):
        return self.folder_root

    def stack_to_string(self):
        return "%s" % [ s.tag for s in self.stack ]

    def find_tag_on_stack(self, tag):
        i = len(self.stack) - 1
        while i >= 0:
            if self.stack[i].tag == tag:
                return self.stack[i]
            i -= 1
        return None

    def implicit_close(self, event, tag):
        
        tag_info = tag_info_dict.get(tag, None)
        if not tag_info:
            return
        
        implicit_close_event = tag_info.get('implicit_close_event', None)
        if not implicit_close_event or not event in implicit_close_event:
            return

        implicit_close_scope = tag_info.get('implicit_close_scope', None)
        implicit_close_tags  = tag_info.get('implicit_close_tags',  None)
        if not (implicit_close_scope or implict_close_tags):
            return
        
        scope_index = len(self.stack) - 1
        while scope_index >= 0:
            if self.stack[scope_index].tag in implicit_close_scope:
                break
            scope_index = scope_index - 1

        i = scope_index + 1
        while i < len(self.stack):
            if self.stack[i].tag in implicit_close_tags:
                break
            i = i + 1

        j = len(self.stack) - 1
        while (j >= i):
            self._handle_endtag(self.stack[j].tag)
            j = j - 1


    def handle_starttag(self, tag, attrs):
        self.implicit_close('begin', tag)
        
        tag_info = tag_info_dict.get(tag, None)
        if not tag_info:
            simple_tag = False
        else:
            simple_tag = tag_info.get('simple_tag', False)
        if not simple_tag:
            top = HTMLTag(tag)
            for attr, value in attrs:
                top.attrs[attr] = value
            self.stack.append(top)

    def _handle_endtag(self, tag):
        top = self.stack.pop();
        if tag == "a":
            bookmark = self.cur_folder.add_bookmark(top.data)
            for attr, value in top.attrs.items():
                bookmark.attrs[attr] = value
            if debug:
                print "%sBookmark %s" % (indent*(len(self.cur_folder.path())),top.data)

        elif top.tag == 'h3' or top.tag == 'h1':
            # Folders are contained in a <DT><H3 attrs>name</H3> sequence
            # Note, this is currently the only use of the H3 tag in a bookmark
            # file so rather than looking for the aforementioned sequence an
            # easy "hack" is to just look for an H3 tag, its attrs, and its
            # data will be the folder name. Note <H1> is reserved for the
            # root folder.
            #
            # Since this is a new folder, we add it as a folder to the
            # currently open folder, it is effectively a push of the folder
            # stack, but we maintain it as simply the currently open folder.
            if top.tag == 'h3':
                self.cur_folder = self.cur_folder.add_folder(top.data)
            else:
                # Tag is h1, must be the root folder
                self.folder_root.reset(top.data, None)
                self.cur_folder = self.folder_root
            for attr, value in top.attrs.items():
                self.cur_folder.attrs[attr] = value
            if debug:
                print "%sPUSH Folder %s" % (indent*(len(self.cur_folder.path())-1),self.cur_folder.name)
        elif top.tag == 'dl':
            # Closing current folder, effectively pop it off the folder stack,
            # the currently open folder is replaced by this folders parent.
            if debug:
                print "%sPOP Folder %s" % (indent*(len(self.cur_folder.path())-1),self.cur_folder.name)
            self.cur_folder = self.cur_folder.parent
        else:
            pass

            


    def handle_endtag(self, tag):
        self.implicit_close('end', tag)
        assert tag == self.stack[-1].tag
        self._handle_endtag(tag)

    def handle_data(self, data):
        tag = self.stack[-1]
        data = data.strip()
        tag.data = tag.data + data

# -----------------------
def visit(entry, type, path, data=None):
    max_len = 80
    level = len(path)-1

    if type == TYPE_FOLDER:
        print "%sFolder: %s(%s) path = [%s]" % (indent*level,
                                                entry.name[0:max_len],
                                                data, entry.path_as_string())
    elif type == TYPE_BOOKMARK:
        print "%sBookmark: %s" % (indent*(level), entry.name[0:max_len])
    elif type == TYPE_FOLDER_END:
        pass
    else:
        raise ValueError

    for attr, value in entry.attrs.items():
        print "%sAttr: %s = %s" % (indent*(level+1), attr, value[0:max_len])

# -----------------------

if __name__ == "__main__":
    bm_root = []
    bm_file = BookmarkHTMLParser()
    bm_file.set_root(bm_root)
    bm_file.feed(open('bookmarks.html').read())
    bm_file.close()

    bm_root_1 = []
    bm_file_1 = BookmarkHTMLParser(bm_root_1)
    bm_file_1.feed(open('bookmarks1.html').read())
    bm_file_1.close()

    if False:
        bm_name = "libical"
        bm_list = bm_file.folder_root.find_bookmark(bm_name)
        if bm_list:
            for bm in bm_list:
                print "found bookmark %s url=%s" % (bm.name, bm.get_attr("href"))
                print "path = %s" % bm.path_as_string()
        else:
            print "%s not found" % bm_name

    if False:
        bm_file.folder_root.traverse(visit)

    if True:
        bm_dict   = bm_file.folder_root.convert_to_dict()
        bm_dict_1 = bm_file_1.folder_root.convert_to_dict()

        dc = util.DictCompare(bm_dict, bm_dict_1)
        dc.compare()
        cs = dc.get_change_set('a', 'b')
        util.dump_change_set(cs)

    if False:
        bm_file.folder_root.write("tmp_bookmarks.html", exclude_attrs=exclude_attrs)




#
# Copyright (C) 2005 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import gconf
import util
import sessionwindow

import debuglog
import errors
from sabayon.lockdown import lockdownapplier

def dprint (fmt, *args):
    debuglog.debug_log (False, debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL, fmt % args)

class LockdownMonitor:
    def __init__ (self, key, handler, data):
        self.gconf_id = None
        self.key = key
        self.handler = handler
        self.data = data

class LockdownApplierSabayon (lockdownapplier.PessulusLockdownApplier):
    def __init__ (self, profile, changes_model):
        self.changes_model = changes_model
        self.source = profile.get_source ("GConf")
        self.client = gconf.client_get_default ()
        self.monitored_keys = {}

        self.changes_model.connect ("changed", self.__changes_model_changed)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL)
    def __changes_model_changed (self, model, change):
        if change == None or change.get_source () != self.source:
            return
        key = change.get_id ()
        if self.monitored_keys.has_key (key):
            for monitor in self.monitored_keys[key]:
                monitor.handler (monitor.data)

    def supports_mandatory_settings (self):
        return True

    def get_schema (self, key):
        return self.client.get_schema (key)

    def __is_mandatory (self, key):
        iter = self.changes_model.find (self.source, key)
        if iter:
            return self.changes_model[iter][sessionwindow.ProfileChangesModel.COLUMN_MANDATORY]
        
        return self.source.get_gconf_key_is_mandatory (key)

    def get_value (self, key):
        iter = self.changes_model.find (self.source, key)
        if iter:
            change = self.changes_model[iter][sessionwindow.ProfileChangesModel.COLUMN_CHANGE]
            val = change.value
        else:
            val = self.client.get (key)
        return val
        
    def get_bool (self, key):
        val = self.get_value (key)
        if val == None:
            val = False
        else:
            val = val.get_bool()
        return (val, self.__is_mandatory (key))

    def get_int (self, key):
        val = self.get_value (key)
        if val != None:
            val = val.get_int()
        return (val, self.__is_mandatory (key))

    def get_string (self, key):
        val = self.get_value (key)
        if val != None:
            val = val.get_string()
        return (val, self.__is_mandatory (key))

    def set_bool (self, key, value, mandatory):
        return self.source.set_gconf_boolean (key, value, mandatory)

    def set_int (self, key, value, mandatory):
        return self.source.set_gconf_int (key, value, mandatory)

    def set_string (self, key, value, mandatory):
        return self.source.set_gconf_string (key, value, mandatory)

    def get_list (self, key, list_type):
        value = self.client.get_list (key, list_type)
        return (value, self.__is_mandatory (key))

    def set_list (self, key, list_type, value, mandatory):
        return self.source.set_gconf_list (key, list_type, value, mandatory)

    def key_is_writable (self, key):
        return True

    def notify_add (self, key, handler, data = None):
        monitor = LockdownMonitor (key, handler, data)

        def __gconf_notify_proxy (client, cnx_id, entry, monitor):
            monitor.handler (monitor.data)
        
        monitor.gconf_id = self.source.add_gconf_notify (key, __gconf_notify_proxy, monitor)
        
        if self.monitored_keys.has_key (key):
            monitors = self.monitored_keys[key]
        else:
            monitors = []
            self.monitored_keys[key] = monitors
        monitors.append(monitor)
            
        return monitor
        
    def notify_remove (self, monitor):
        self.source.remove_gconf_notify (monitor.gconf_id)
        monitors = self.monitored_keys[monitor.key]
        monitors.remove (monitor)

    # We always monitor all dirs anyway
    def add_dir (self, dir, preloadtype):
        pass
    def remove_dir (self, dir):
        pass
        



#
# Copyright (C) 2005 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os
import os.path
import sys
import shutil
import tempfile
import pwd
import gobject
import gtk
import gtk.glade
import storage
import editorwindow
import usersdialog
import util
import userdb
import protosession
import debuglog
import errors
from config import *

def dprint (fmt, *args):
    debuglog.debug_log (False, debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL, fmt % args)

def mprint (fmt, *args):
    debuglog.debug_log (True, debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL, fmt % args)

def _get_profile_path_for_name (profile_name):
    return os.path.join (PROFILESDIR, profile_name + ".zip")

class Session (gobject.GObject):
    __gsignals__ = {
        "finished" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ())
        }

    def __init__ (self, username, profile_name):
        gobject.GObject.__init__ (self)

        self.username     = username
        self.profile_name = profile_name
        self.profile_path = _get_profile_path_for_name (profile_name)

        self.pw = pwd.getpwnam (self.username)

        self.user_profile_path = None
        self.temp_homedir      = None

        self.session_pid             = 0
        self.session_child_watch     = 0
        self.session_stderr          = None
        self.session_stderr_watch_id = 0
        self.session_log_str         = ""

    def __del__ (self):
        if self.temp_homedir:
            protosession.reset_shell_and_homedir (self.username, self.temp_homedir)
            self.temp_homedir = None
        if self.user_profile_path:
            os.remove (self.user_profile_path)
            self.user_profile_path = None
        if self.temp_xauth_path:
            os.remove (self.temp_xauth_path)
            self.temp_xauth_path = None

    def __copy_xauthority (self):
        if not os.environ.has_key ("XAUTHORITY"):
            return None

        xauthority = os.environ["XAUTHORITY"]

        (fd, temp_path) = tempfile.mkstemp (prefix = "xauth-%s-" % self.username)
        os.close (fd)

        shutil.copy2 (xauthority, temp_path)

        os.chown (temp_path, self.pw.pw_uid, self.pw.pw_gid)

        dprint ("Copied $XAUTHORITY (%s) temporarily to %s", xauthority, temp_path)

        return temp_path

    def __copy_to_user (self, profile_path):
        (fd, user_path) = tempfile.mkstemp (prefix = "profile-%s-" % self.username, suffix = ".zip")
        os.close (fd)

        shutil.copy2 (profile_path, user_path)

        os.chown (user_path, self.pw.pw_uid, self.pw.pw_gid)

        dprint ("Copied %s temporarily to %s", profile_path, user_path)

        return user_path

    def __copy_from_user (self, user_path, profile_path):
        os.chown (user_path, os.geteuid (), os.getegid ())
        shutil.move (user_path, profile_path)
        dprint ("Moved %s back from %s", user_path, profile_path)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL)
    def __session_child_watch_handler (self, pid, status):
        if not os.WIFEXITED (status):
            exit_code = util.EXIT_CODE_FATAL

            if os.WIFSIGNALED (status):
                signal_num = os.WTERMSIG (status)

                mprint ("sabayon-session exited with SIGNAL %s", signal_num)
            else:
                mprint ("sabayon-session exited for an unknown reason")
        else:
            exit_code = os.WEXITSTATUS (status)

            # Here we cannot throw an exception for the error cases,
            # since we are a callback running out of the Glib main
            # loop.  So we have to set a flag and let the code
            # *outside* the main loop pick it up.

            if exit_code == util.EXIT_CODE_NORMAL:
                mprint ("sabayon-session exited normally")
                success = True
            elif exit_code == util.EXIT_CODE_RECOVERABLE:
                errors.errors_log_recoverable_error (debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL,
                                                     "sabayon-session exited with RECOVERABLE exit status")
                # FIXME: throw a warning dialog
            else:
                errors.errors_log_fatal_error (debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL,
                                               "sabayon-session exited with a FATAL ERROR (exit code %s)" % exit_code)
                gtk.main_quit () # so that the toplevel 'sabayon' will exit the main loop and show the fatal error

        protosession.clobber_user_processes (self.username)
        protosession.reset_shell_and_homedir (self.username, self.temp_homedir)
        self.temp_homedir = None

        if self.temp_xauth_path:
            os.remove (self.temp_xauth_path)
            self.temp_xauth_path = None

        self.__copy_from_user (self.user_profile_path, self.profile_path)
        self.user_profile_path = None

        self.session_pid         = 0
        self.session_child_watch = 0

        gobject.source_remove (self.session_stderr_watch_id)
        self.session_stderr_watch_id = 0

        self.session_stderr.close ()
        self.session_stderr = None

        self.emit ("finished")

        return False

    def build_envp_for_child (self):
        # gobject.spawn_async() wants a sequence, not a dictionary, so we build a sequence...
        new_environ = []
        for key in PASSTHROUGH_ENVIRONMENT:
            if os.environ.has_key (key):
                new_environ.append ("%s=%s" % (key, os.environ[key]))

        new_environ = new_environ + ["PATH=%s"     % DEFAULT_PATH,
                                     "SHELL=%s"    % DEFAULT_SHELL,
                                     "DISPLAY=%s"  % os.environ["DISPLAY"],
                                     "HOME=%s"     % self.temp_homedir,
                                     "LOGNAME=%s"  % self.pw.pw_name,
                                     "USER=%s"     % self.pw.pw_name,
                                     "USERNAME=%s" % self.pw.pw_name]

        if self.temp_xauth_path:
            new_environ.append ("XAUTHORITY=%s" % self.temp_xauth_path)

        return new_environ

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL)
    def session_stderr_io_cb (self, source_fd, condition, session):
        if condition & gobject.IO_IN:
            s = session.session_stderr.read ()
            session.session_log_str = session.session_log_str + s
#            print "%s: got from sabayon-session stderr: \n<BEGIN SABAYON-SESSION STDERR>\n%s\n<END SABAYON-SESSION STDERR>" % (os.getpid (), s)

        if condition & gobject.IO_HUP:
            mprint ("========== BEGIN SABAYON-SESSION LOG ==========\n"
                    "%s\n"
                    "========== END SABAYON-SESSION LOG ==========",
                    session.session_log_str)
            return False

        return True

    def start (self):
        self.user_profile_path = self.__copy_to_user (self.profile_path)
        self.temp_homedir = protosession.setup_shell_and_homedir (self.username)
        protosession.clobber_user_processes (self.username)

        display_number = protosession.find_free_display ()

        self.temp_xauth_path = self.__copy_xauthority ()

        def child_setup_fn (self):
            os.setgid (self.pw.pw_gid)
            os.setuid (self.pw.pw_uid)
            os.setsid ()
            os.umask (022)

        # FIXME: get_readable_log_config_filename() doesn't work here.
        # Create a temporary copy of the log config file and use *that*.
        argv = SESSION_TOOL_ARGV + [ ("--admin-log-config=%s" % util.get_admin_log_config_filename ()),
                                     ("--readable-log-config=%s" % util.get_readable_log_config_filename ()),
                                     self.profile_name,
                                     self.user_profile_path,
                                     str (display_number) ]
        envp = self.build_envp_for_child ()
        cwd = self.temp_homedir

        # FIXME: do we need any special processing if this throws an exception?
        # We'll catch it in the toplevel and exit with a fatal error code, anyway.
        (pid, oink, oink, stderr_fd) = gobject.spawn_async (argv, envp, cwd,
                                                            gobject.SPAWN_DO_NOT_REAP_CHILD,
                                                            child_setup_fn, self,
                                                            None, None, True)	# stdin, stdout, stderr

        self.session_pid = pid;
        self.session_stderr = os.fdopen (stderr_fd)
        self.session_stderr_watch_id = gobject.io_add_watch (stderr_fd,
                                                             gobject.IO_IN | gobject.IO_HUP,
                                                             self.session_stderr_io_cb, self)
        self.session_child_watch = gobject.child_watch_add (self.session_pid,
                                                            self.__session_child_watch_handler)

gobject.type_register (Session)

class ProfilesModel (gtk.ListStore):
    (
        COLUMN_NAME,
    ) = range (1)

    def __init__ (self):
        gtk.ListStore.__init__ (self, str)
        self.reload ()

    def reload (self):
        self.clear ()
        profiles = userdb.get_database ().get_profiles ()
        profiles.sort ()
        for profile in profiles:
            self.set (self.append (),
                      self.COLUMN_NAME, profile)

class AddProfileDialog:
    def __init__ (self, profiles_model):
        self.profiles_model = profiles_model

        glade_file = os.path.join (GLADEDIR, "sabayon.glade")
        self.xml = gtk.glade.XML (glade_file, "add_profile_dialog", PACKAGE)

        self.dialog = self.xml.get_widget ("add_profile_dialog")
        self.dialog.connect ("destroy", gtk.main_quit)
        self.dialog.set_default_response (gtk.RESPONSE_ACCEPT)
        self.dialog.set_icon_name ("sabayon")

        self.add_button = self.xml.get_widget ("add_profile_add_button")
        self.add_button.set_sensitive (False)

        self.name_entry = self.xml.get_widget ("add_profile_name_entry")
        self.name_entry.connect ("changed", self.__name_entry_changed)
        self.name_entry.set_activates_default (True)

        self.base_combo = self.xml.get_widget ("add_profile_base_combo")
        self.base_combo.set_model (self.profiles_model)
        if self.profiles_model.get_iter_first () is None:
            self.base_combo.set_sensitive (False)

        renderer = gtk.CellRendererText ()
        self.base_combo.pack_start (renderer, True)
        self.base_combo.set_attributes (renderer, text = ProfilesModel.COLUMN_NAME)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __name_entry_changed (self, entry):
        text = entry.get_text ()
        if not text or text.isspace ():
            self.add_button.set_sensitive (False)
        else:
            self.add_button.set_sensitive (True)

    def run (self, parent):
        self.name_entry.grab_focus ()
        self.dialog.set_transient_for (parent)
        self.dialog.present ()
        response = self.dialog.run ()
        self.dialog.hide ()

        if response != gtk.RESPONSE_ACCEPT:
            return (None, None)

        row = self.base_combo.get_active_iter ()
        if row:
            base = self.profiles_model.get_value (row, ProfilesModel.COLUMN_NAME)
        else:
            base = None

        return (self.name_entry.get_text (), base)

class ProfilesDialog:
    def __init__ (self):
        assert os.geteuid () == 0

        glade_file = os.path.join (GLADEDIR, "sabayon.glade")
        self.xml = gtk.glade.XML (glade_file, "profiles_dialog", PACKAGE)

        self.dialog = self.xml.get_widget ("profiles_dialog")
        self.dialog.connect ("destroy", gtk.main_quit)
        self.dialog.set_default_response (gtk.RESPONSE_ACCEPT)
        self.dialog.set_icon_name ("sabayon")

        self.profiles_list = self.xml.get_widget ("profiles_list")
        self.__setup_profiles_list ()

        self.profiles_list.connect ("key-press-event", self.__handle_key_press)

        self.add_button = self.xml.get_widget ("add_button")
        self.add_button.connect ("clicked", self.__add_button_clicked)

        self.remove_button = self.xml.get_widget ("remove_button")
        self.remove_button.connect ("clicked", self.__remove_button_clicked)

        self.edit_button = self.xml.get_widget ("edit_button")
        self.__fix_button_align (self.edit_button)
        self.edit_button.connect ("clicked", self.__edit_button_clicked)

        self.details_button = self.xml.get_widget ("details_button")
        self.__fix_button_align (self.details_button)
        self.details_button.connect ("clicked", self.__details_button_clicked)

        self.users_button = self.xml.get_widget ("users_button")
        self.__fix_button_align (self.users_button)
        self.users_button.connect ("clicked", self.__users_button_clicked)

        self.help_button = self.xml.get_widget ("help_button")
        self.help_button.hide ()

        self.dialog.connect ("response", self.__dialog_response)

        (width, height) = self.profiles_list.size_request ()

        self.dialog.set_default_size (min (width + 250, 450),
                                      min (height + 190, 400))

        self.profiles_list.grab_focus ()
        self.__profile_selection_changed (self.profiles_list.get_selection ())

        self.dialog.show ()

    def __fix_button_align (self, button):
        child = button.get_child ()

        if isinstance (child, gtk.Alignment):
            child.set_property ("xalign", 0.0)
        elif isinstance (child, gtk.Label):
            child.set_property ("xalign", 0.0)

    def __setup_profiles_list (self):
        self.profiles_model = ProfilesModel ()
        self.profiles_list.set_model (self.profiles_model)

        self.profiles_list.get_selection ().set_mode (gtk.SELECTION_SINGLE)
        self.profiles_list.get_selection ().connect ("changed", self.__profile_selection_changed)

        c = gtk.TreeViewColumn (_("Name"),
                                gtk.CellRendererText (),
                                text = ProfilesModel.COLUMN_NAME)
        self.profiles_list.append_column (c)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __dialog_response (self, dialog, response_id):
        dialog.destroy ()

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __add_button_clicked (self, button):
        (profile_name, base_profile) = AddProfileDialog (self.profiles_model).run (self.dialog)
        if profile_name:
            self.__create_new_profile (profile_name, base_profile)

    def __get_selected_profile (self):
        (model, row) = self.profiles_list.get_selection ().get_selected ()
        if not row:
            return None
        return model[row][ProfilesModel.COLUMN_NAME]

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL)
    def __session_finished (self, session):
        debuglog.uprint ("Finishing editing profile")
        self.dialog.set_sensitive (True)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __edit_button_clicked (self, button):
        profile_name = self.__get_selected_profile ()
        if profile_name:
            self.dialog.set_sensitive (False)

            session = Session (PROTOTYPE_USER, profile_name)
            session.connect ("finished", self.__session_finished)
            debuglog.uprint ("Starting to edit profile '%s'", profile_name)
            session.start ()

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __details_button_clicked (self, button):
        profile_name = self.__get_selected_profile ()
        if profile_name:
            editorwindow.ProfileEditorWindow (profile_name, self.dialog)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __users_button_clicked (self, button):
        profile_name = self.__get_selected_profile ()
        if profile_name:
            usersdialog.UsersDialog (profile_name, self.dialog)

    def __delete_currently_selected (self):
        (model, selected) = self.profiles_list.get_selection ().get_selected ()
        if selected:
            if model.iter_next (selected):
                select = model[model.iter_next (selected)][ProfilesModel.COLUMN_NAME]
            else:
                select = None
                row = model.get_iter_first ()
                while row and model.iter_next (row):
                    next = model.iter_next (row)
                    if model.get_string_from_iter (next) == model.get_string_from_iter (selected):
                        select = model[row][ProfilesModel.COLUMN_NAME]
                        break
                    row = next

            profile_name = model[selected][ProfilesModel.COLUMN_NAME]
            dprint ("Deleting '%s'", profile_name)
            os.remove (_get_profile_path_for_name (profile_name))

            db = userdb.get_database ()
            if db.get_default_profile (False) == profile_name:
                db.set_default_profile (None)
            for user in db.get_users ():
                if db.get_profile (user, False, True) == profile_name:
                    db.set_profile (user, None)

            self.profiles_model.reload ()

            row = None
            if select:
                row = self.profiles_model.get_iter_first ()
                while row:
                    if select == model[row][ProfilesModel.COLUMN_NAME]:
                        break
                    row = model.iter_next (row)
            if not row:
                row = self.profiles_model.get_iter_first ()
            if row:
                self.profiles_list.get_selection ().select_iter (row)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __remove_button_clicked (self, button):
        self.__delete_currently_selected ()

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __handle_key_press (self, profiles_list, event):
        if event.keyval in (gtk.keysyms.Delete, gtk.keysyms.KP_Delete):
            self.__delete_currently_selected ()

    def __make_unique_profile_name (self, profile_name):
        profiles = userdb.get_database ().get_profiles ()

        name = profile_name
        idx = 1
        while name in profiles:
            #
            # Translators: this string specifies how a profile
            #              name is concatenated with an integer
            #              to form a unique profile name e.g.
            #              "Artist Workstation (5)"
            #
            name = _("%s (%s)") % (profile_name, idx)
            idx += 1

        return name

    def __create_new_profile (self, profile_name, base_profile):
        profile_name = self.__make_unique_profile_name (profile_name)

        if base_profile:
            base_storage = storage.ProfileStorage (base_profile)
            new_storage = base_storage.copy (profile_name)
        else:
            new_storage = storage.ProfileStorage (profile_name)
            new_storage.save ()

        self.profiles_model.reload ()
        row = self.profiles_model.get_iter_first ()
        while row:
            if self.profiles_model[row][ProfilesModel.COLUMN_NAME] == profile_name:
                self.profiles_list.get_selection ().select_iter (row)
                return
            row = self.profiles_model.iter_next (row)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __profile_selection_changed (self, selection):
        profile_name = self.__get_selected_profile ()
        self.edit_button.set_sensitive (profile_name != None)
        self.details_button.set_sensitive (profile_name != None)
        self.users_button.set_sensitive (profile_name != None)
        self.remove_button.set_sensitive (profile_name != None)



#!/usr/bin/env python

# vim: set ts=4 sw=4 et:

#
# Copyright (C) 2005 Vincent Untz <vuntz@gnome.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#

import gobject
import gtk
import sys

from sabayon import errors
from sabayon import debuglog

try:
    set
except:
    from sets import Set as set

class PessulusSimpleEditableTreeview (gobject.GObject):
    (
        COLUMN_EDITABLE,
    ) = range (1)

    __gsignals__ = {
        "changed" : ( gobject.SIGNAL_RUN_LAST, None, (gobject.TYPE_PYOBJECT,) )
    }

    def __init__ (self, treeview, addbutton, editbutton, removebutton, sort = True, strip_text = True):
        gobject.GObject.__init__(self)

        self.content_set = None
        self.selected_content = None
        self.new_edited_path = None
        self.editing = False
        self.sensitive = True
        self.strip_text = strip_text

        self.treeview = treeview
        self.addbutton = addbutton
        self.editbutton = editbutton
        self.removebutton = removebutton

        self.liststore = gtk.ListStore (str)
        self.treeview.set_model (self.liststore)
        if sort:
            self.liststore.set_sort_column_id (self.COLUMN_EDITABLE,
                                               gtk.SORT_ASCENDING)

        self.__create_columns ()

        self.addbutton.connect ("clicked", self.__on_add_button_clicked)
        self.editbutton.connect ("clicked", self.__on_edit_button_clicked)
        self.removebutton.connect ("clicked", self.__on_remove_button_clicked)

        treeselection = self.treeview.get_selection ()
        treeselection.set_mode (gtk.SELECTION_SINGLE)
        treeselection.connect ("changed", self.__on_treeselection_changed)

        self.__update_sensitivity ()

    def __create_columns (self):
        self.column = gtk.TreeViewColumn ()
        self.treeview.append_column (self.column)

        self.cell = gtk.CellRendererText ()
        self.column.pack_start (self.cell, True)
        self.column.set_attributes (self.cell, text = self.COLUMN_EDITABLE)

        self.cell.set_property("editable", True)
        self.cell.connect("edited", self.__on_cell_edited)
        # we want to know if we're editing or not
        self.cell.connect("editing-started", self.__on_cell_editing_started)
        # we need to connect to this signal since if we add a row
        # and don't edit it, it shouldn't be really added
        self.cell.connect("editing-canceled", self.__on_cell_editing_canceled)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __on_cell_edited (self, cell, path, new_text):
        debuglog.uprint ('PessulusSimpleEditableTreeview: edited cell; new contents: "%s"', new_text)

        self.editing = False
        self.__update_sensitivity ()

        new = False
        if self.new_edited_path:
            if self.new_edited_path != path:
                new = True
            else:
                print >> sys.stderr, "Warning: path should have been a new edited one in the treeview"
            self.new_edited_path = None

        if self.strip_text:
            text = new_text.strip ()
        else:
            text = new_text

        # don't accept new items that are ""
        if text == "" and new:
            del (self.liststore[path])
            return

        if new or self.liststore[path][self.COLUMN_EDITABLE] != text:
            # save the selected content so that it will be selected again
            # later
            self.selected_content = text

            if not new:
                self.content_set.remove (self.liststore[path][self.COLUMN_EDITABLE])
            if text != "":
                self.content_set.add (text)

            self.__update_model ()
            self.emit ("changed", self.content_set)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __on_cell_editing_started (self, cell, editable, path):
        self.editing = True
        self.__update_sensitivity ()

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __on_cell_editing_canceled (self, cell):
        self.editing = False
        self.__update_sensitivity ()

        if self.new_edited_path:
            del (self.liststore[self.new_edited_path])
            self.new_edited_path = None

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __on_add_button_clicked (self, button):
        debuglog.uprint ("PessulusSimpleEditableTreeview: add button clicked")

        # add a row and start editing it
        iter = self.liststore.append ()
        path = self.liststore.get_path (iter)
        self.treeview.set_cursor_on_cell (path, self.column, self.cell, True)
        self.new_edited_path = path

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __on_edit_button_clicked (self, button):
        debuglog.uprint ("PessulusSimpleEditableTreeview: edit button clicked")

        model, iter = self.treeview.get_selection ().get_selected ()

        # if nothing selected...
        if not iter:
            print >> sys.stderr, "Warning: ask for edition in treeview while nothing selected"
            return

        path = model.get_path (iter)
        self.treeview.set_cursor_on_cell (path, self.column, self.cell, True)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __on_remove_button_clicked (self, button):
        debuglog.uprint ("PessulusSimpleEditableTreeview: remove button clicked")

        model, iter = self.treeview.get_selection ().get_selected ()

        # if nothing selected...
        if not iter:
            print >> sys.stderr, "Warning: ask for removal in treeview while nothing selected"
            return

        selected_value = model[iter][self.COLUMN_EDITABLE]

        # if selection should not be here...
        if selected_value not in self.content_set:
            print >> sys.stderr, "Warning: %s should not be in the treeview" % selected_value
            del (model[iter])
            return
        
        # try to find the future selection (after this item has been removed)
        # note that we can not save a path since the paths might change
        new_selected_iter = model.iter_next (iter)
        if not new_selected_iter:
            # there's no next item. This item was the last one. Select the
            # item that's before.
            children = model.iter_n_children (None)
            if children > 1:
                new_selected_iter = model.iter_nth_child (None, children - 2)
        if new_selected_iter:
            path = model.get_path (new_selected_iter)
            self.selected_content = model[path][self.COLUMN_EDITABLE]

        self.content_set.remove (selected_value)
        self.__update_model ()
        self.emit ("changed", self.content_set)

    @errors.checked_callback (debuglog.DEBUG_LOG_DOMAIN_USER)
    def __on_treeselection_changed (self, treeselection):
        self.__update_sensitivity ()

    def __update_sensitivity (self):
        self.treeview.set_sensitive (self.sensitive)

        sensitive = self.sensitive and not self.editing
        self.addbutton.set_sensitive (sensitive)

        selected = self.treeview.get_selection ().count_selected_rows () > 0

        sensitive = self.sensitive and not self.editing and selected
        self.editbutton.set_sensitive (sensitive)

        sensitive = self.sensitive and selected
        self.removebutton.set_sensitive (sensitive)

    def __update_model (self):
        def select_if_should (model, path, iter, data):
            if self.selected_content == model[path][data.COLUMN_EDITABLE]:
                data.treeview.get_selection ().select_path (path)
                data.treeview.scroll_to_cell (path)
                return True

        self.liststore.clear ()

        for value in self.content_set:
            iter = self.liststore.append ()
            self.liststore.set (iter, self.COLUMN_EDITABLE, value)

        if self.selected_content:
            self.liststore.foreach (select_if_should, self)
            self.selected_content = None

        self.__update_sensitivity ()

    def set_sensitive (self, sensitive):
        self.sensitive = sensitive
        self.__update_sensitivity ()

    def update_set (self, set):
        if set == self.content_set:
            return

        model, iter = self.treeview.get_selection ().get_selected ()
        if iter:
            # save the selected content so that it will be selected again
            # later
            self.selected_content = model[iter][self.COLUMN_EDITABLE]

        self.content_set = set.copy ()
        self.__update_model ()

if gtk.pygtk_version < (2, 8, 0):
    gobject.type_register (PessulusSimpleEditableTreeview)



#
# Copyright (C) 2005 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import gtk

class FileViewer (gtk.Window):
    def __init__ (self, path, description, parent_window):
        gtk.Window.__init__ (self, gtk.WINDOW_TOPLEVEL)

        self.path = path
        
        self.set_title (_("Profile file: %s")%description)
        self.set_icon_name ("sabayon")
        self.set_transient_for (parent_window)
        self.set_destroy_with_parent (True)
        self.set_default_size (480, 380)

        self.scrolled = gtk.ScrolledWindow ()
        self.scrolled.set_policy (gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.scrolled.set_shadow_type (gtk.SHADOW_IN)
        self.scrolled.show ()
        self.add (self.scrolled)

        self.__setup_textview ()
    
    def __setup_textview (self):
        self.textview = gtk.TextView ()
        self.textview.set_editable (False)
        self.textview.set_wrap_mode (gtk.WRAP_WORD)
        self.textview.show()
        
        self.scrolled.add (self.textview)

        buffer = self.textview.get_buffer ()
        buffer.set_text (file (self.path).read ())



#!/usr/bin/env python

# vim: set ts=4 sw=4 et:

#
# Copyright (C) 2005 Vincent Untz <vuntz@gnome.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#

class PessulusLockdownApplier:
    def __init__ (self):
        raise NotImplementedError

    def supports_mandatory_settings (self):
        raise NotImplementedError

    def get_schema (self, key):
        raise NotImplementedError

    def get_bool (self, key):
        raise NotImplementedError

    def set_bool (self, key, value, mandatory):
        raise NotImplementedError

    def get_list (self, key, list_type):
        raise NotImplementedError

    def set_list (self, key, list_type, value, mandatory):
        raise NotImplementedError

    def key_is_writable (self, key):
        raise NotImplementedError

    def notify_add (self, key, handler, data = None):
        raise NotImplementedError

    def notify_remove (self, monitor):
        raise NotImplementedError

    def add_dir (self, dir, preloadtype):
        raise NotImplementedError

    def remove_dir (self, dir):
        raise NotImplementedError



#
# Copyright (C) 2005 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import gobject
import gtk
import gtk.gdk
import xlib
import util
import debuglog

def dprint (fmt, *args):
    debuglog.debug_log (False, debuglog.DEBUG_LOG_DOMAIN_ADMIN_TOOL, fmt % args)
    
class SessionWidget (gtk.Widget):
    # FIXME:
    #   Should be able override do_size_allocate() without this
    #   See bug #308099
    __gsignals__ = { 'size-allocate' : 'override' }
    
    def __init__ (self, session_width, session_height):
        gtk.Widget.__init__ (self)
        
        self.set_flags (self.flags() | gtk.NO_WINDOW)
        self.set_property ("can-focus", True)

        self.session_width  = session_width
        self.session_height = session_height

    def do_size_request (self, requisition):
        focus_width = self.style_get_property ("focus-line-width")
        focus_pad   = self.style_get_property ("focus-padding")
        
        requisition.width  = self.session_width  + 2 * focus_width + 2 * focus_pad
        requisition.height = self.session_height + 2 * focus_width + 2 * focus_pad

    def __calculate_position (self, allocation):
        focus_width = self.style_get_property ("focus-line-width")
        focus_pad   = self.style_get_property ("focus-padding")

        x = allocation.x + focus_width + focus_pad
        y = allocation.y + focus_width + focus_pad
            
        alloc_width  = allocation.width  - 2 * focus_width - 2 * focus_pad
        alloc_height = allocation.height - 2 * focus_width - 2 * focus_pad

        x += max (0, (alloc_width  - self.session_width)  / 2)
        y += max (0, (alloc_height - self.session_height) / 2)

        return (x, y)

    def do_size_allocate (self, allocation):
        self.allocation = allocation

        if self.flags() & gtk.REALIZED:
            (x, y) = self.__calculate_position (allocation)
            
            self.session_window.move_resize (x, y, self.session_width, self.session_height)
            self.input_window.move_resize   (x, y, self.session_width, self.session_height)

    def do_realize (self):
        gtk.Widget.do_realize (self)

        (x, y) = self.__calculate_position (self.allocation)

        self.session_window = gtk.gdk.Window (parent = self.get_parent_window (),
                                              window_type = gtk.gdk.WINDOW_CHILD,
                                              x = x,
                                              y = y,
                                              width  = self.session_width,
                                              height = self.session_height,
                                              wclass = gtk.gdk.INPUT_OUTPUT,
                                              visual = self.get_visual (),
                                              colormap = self.get_colormap (),
                                              event_mask = gtk.gdk.BUTTON_RELEASE_MASK)
        self.session_window.set_user_data (self)

        event_mask = gtk.gdk.BUTTON_PRESS_MASK   | \
                     gtk.gdk.BUTTON_RELEASE_MASK | \
                     gtk.gdk.POINTER_MOTION_MASK | \
                     gtk.gdk.ENTER_NOTIFY_MASK   | \
                     gtk.gdk.LEAVE_NOTIFY_MASK   | \
                     gtk.gdk.KEY_PRESS_MASK      | \
                     gtk.gdk.KEY_RELEASE_MASK;

        self.input_window = gtk.gdk.Window (parent = self.get_parent_window (),
                                            window_type = gtk.gdk.WINDOW_CHILD,
                                            x = x,
                                            y = y,
                                            width  = self.session_width,
                                            height = self.session_height,
                                            wclass = gtk.gdk.INPUT_ONLY,
                                            visual = self.get_visual (),
                                            colormap = self.get_colormap (),
                                            event_mask = event_mask)
        self.input_window.set_user_data (self)

    def do_unrealize (self):
        # FIXME:
        #   Causes a warning with pygtk 2.6.2
        #   See bug #308384
        self.input_window.set_user_data (None)
        self.input_window.destroy ()
        
        self.session_window.set_user_data (None)
        self.session_window.destroy ()
        
        gtk.Widget.do_unrealize (self)

    def do_map (self):
        gtk.Widget.do_map (self)
        self.session_window.show ()
        self.input_window.show ()

    def do_unmap (self):
        self.input_window.hide ()
        self.session_window.hide ()
        gtk.Widget.do_unmap (self)

    def do_expose_event (self, event):
        if self.flags() & gtk.HAS_FOCUS:
            focus_width = self.style_get_property ("focus-line-width")
            focus_pad   = self.style_get_property ("focus-padding")
            
            (x, y) = self.__calculate_position (self.allocation)

            self.style.paint_focus (self.window,
                                    self.state,
                                    event.area,
                                    self,
                                    "sabayon-session",
                                    x - focus_width - focus_pad,
                                    y - focus_width - focus_pad,
                                    self.session_width  + 2 * focus_width + 2 * focus_pad,
                                    self.session_height + 2 * focus_width + 2 * focus_pad)
        return False

    def __update_input_only_window (self):
        if not self.flags() & gtk.REALIZED:
            return
        
        if self.flags() & gtk.HAS_FOCUS:
            self.input_window.hide ()
        else:
            self.input_window.show ()
            
    def do_focus_in_event (self, event):
        self.__update_input_only_window ()
        return gtk.Widget.do_focus_in_event (self, event)

    def do_focus_out_event (self, event):
        self.__update_input_only_window ()
        return gtk.Widget.do_focus_in_event (self, event)

    def do_button_press_event (self, event):
        if not self.flags() & gtk.HAS_FOCUS:
            self.grab_focus ()
            
        if not event.send_event:
            dprint ("Resending button press; button = %d, state = 0x%x", event.button, event.state);
            xlib.send_button_event (self.session_window, True, event.time, event.button, event.state);
        
        return True
    
    def do_button_release_event (self, event):
        if not event.send_event:
            dprint ("Resending button release; button = %d, state = 0x%x", event.button, event.state);
            xlib.send_button_event (self.session_window, False, event.time, event.button, event.state);
        
        return True

    def do_motion_notify_event (self, event):
        if not event.send_event:
            dprint ("Resending motion notify; x = %d, y = %d", event.x, event.y);
            xlib.send_motion_event (self.session_window, event.time, event.x, event.y);
        
        return True
    
    def do_enter_notify_event (self, event):
        if not event.send_event:
            dprint ("Resending enter notify; x = %d, y = %d, detail = %d", event.x, event.y, event.detail);
            xlib.send_crossing_event (self.session_window, True, event.time, event.x, event.y, event.detail);
        
        return True
    
    def do_leave_notify_event (self, event):
        if not event.send_event:
            dprint ("Resending leave notify; x = %d, y = %d, detail = %d", event.x, event.y, event.detail);
            xlib.send_crossing_event (self.session_window, False, event.time, event.x, event.y, event.detail);
        
        return True
    
    def do_key_press_event (self, event):
        if not event.send_event:
            dprint ("Resending key press; keycode = 0x%x, state = 0x%x", event.hardware_keycode, event.state)
            xlib.send_key_event (self.session_window, True, event.time, event.hardware_keycode, event.state)
        
        return True

    def do_key_release_event (self, event):
        if not event.send_event:
            dprint ("Resending key release; keycode = 0x%x, state = 0x%x", event.hardware_keycode, event.state)
            xlib.send_key_event (self.session_window, False, event.time, event.hardware_keycode, event.state)
        
        return True

gobject.type_register (SessionWidget)



#
# Copyright (C) 2005 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os
import os.path
import tempfile
import shutil
import util
from config import *
import debuglog

def dprint (fmt, *args):
    debuglog.debug_log (False, debuglog.DEBUG_LOG_DOMAIN_USERMOD, fmt % args)

def set_shell (username, shell):
    argv = USERMOD_ARGV + [ "-s", shell, username ]
    dprint ("Executing %s" % argv)
    util.uninterruptible_spawnv (os.P_WAIT, argv[0], argv)

#
# FIXME:
#  we're fairly screwed if there's another gamin, gconfd-2
#  or whatever already running when we do this. We probably
#  should just shut them down.
#
def set_homedir (username, homedir):
    argv = USERMOD_ARGV + [ "-d", homedir, username ]
    dprint ("Executing %s" % argv)
    util.uninterruptible_spawnv (os.P_WAIT, argv[0], argv)

def create_temporary_homedir (uid, gid):
    temp_homedir = tempfile.mkdtemp (prefix = "sabayon-temp-home-")

    def copy_tree (src, dst, uid, gid):
        for file in os.listdir (src):
            src_path = os.path.join (src, file)
            dst_path = os.path.join (dst, file)

            if os.path.islink (src_path):
                linkto = os.readlink (src_path)
                os.symlink (linkto, dst_path)
            elif os.path.isdir (src_path):
                os.mkdir (dst_path)
                copy_tree (src_path, dst_path, uid, gid)
            else:
                shutil.copy2 (src_path, dst_path)
            
            os.chown (dst_path, uid, gid)

    copy_tree (SKEL_HOMEDIR, temp_homedir, uid, gid)
    os.chown (temp_homedir, uid, gid)
    return temp_homedir
